"""
This file (./graphique_doctest.sage) was *autogenerated* from ./graphique.tex,
with sagetex.sty version 2011/05/27 v2.3.1.
It contains the contents of all the sageexample environments from this file.
You should be able to doctest this file with:
sage -t ./graphique_doctest.sage
It is always safe to delete this file; it is not used in typesetting your
document.

Sage example in ./graphique.tex, line 18::

  sage: reset()

Sage example in ./graphique.tex, line 61::

  sage: plot(x * sin(1/x), x, -2, 2, plot_points=500)
  Graphics object consisting of 1 graphics primitive

Sage example in ./graphique.tex, line 156::

  sage: def p(x, n):
  ....:     return(taylor(sin(x), x, 0, n))
  sage: xmax = 15 ; n = 15
  sage: g = plot(sin(x), x, -xmax, xmax)
  sage: for d in range(n):  # long time
  ....:   g += plot(p(x, 2 * d + 1), x, -xmax, xmax,\
  ....:     color=(1.7*d/(2*n), 1.5*d/(2*n), 1-3*d/(4*n)))
  sage: g.show(ymin=-2, ymax=2)

Sage example in ./graphique.tex, line 237::

  sage: f2(x) = 1; f1(x) = -1
  sage: f = piecewise([[(-pi,0),f1],[(0,pi),f2]])
  sage: S = f.fourier_series_partial_sum(20,pi)
  sage: g = plot(S, x, -8, 8, color='blue')
  sage: saw(x) = x - 2 * pi * floor((x + pi) / (2 * pi))
  sage: g += plot(saw(x) / abs(saw(x)), x, -8, 8, color='red')
  sage: g
  Graphics object consisting of 2 graphics primitives

Sage example in ./graphique.tex, line 311::

  sage: t = var('t')
  sage: x = cos(t) + cos(7*t)/2 + sin(17*t)/3
  sage: y = sin(t) + sin(7*t)/2 + cos(17*t)/3
  sage: g = parametric_plot((x, y), (t, 0, 2*pi))
  sage: g.show(aspect_ratio=1)

Sage example in ./graphique.tex, line 364::

  sage: t = var('t'); n = 20/19
  sage: g1 = polar_plot(1+2*cos(n*t),(t,0,n*36*pi),plot_points=5000)
  sage: g2 = polar_plot(1+1/3*cos(n*t),(t,0,n*36*pi),plot_points=5000)
  sage: g1.show(aspect_ratio=1); g2.show(aspect_ratio=1)

Sage example in ./graphique.tex, line 500::

  sage: bar_chart([randrange(15) for i in range(20)])
  Graphics object consisting of 1 graphics primitive
  sage: bar_chart([x^2 for x in range(1,20)], width=0.2)
  Graphics object consisting of 1 graphics primitive

Sage example in ./graphique.tex, line 550::

  sage: liste = [10 + floor(10*sin(i)) for i in range(100)]
  sage: bar_chart(liste)
  Graphics object consisting of 1 graphics primitive
  sage: TimeSeries(liste).plot_histogram(bins=20)
  Graphics object consisting of 20 graphics primitives

Sage example in ./graphique.tex, line 714::

  sage: n, l, x, y = 10000, 1, 0, 0; p = [[0, 0]]
  sage: for k in range(n):
  ....:     theta = (2 * pi * random()).n(digits=5)
  ....:     x, y = x + l * cos(theta), y + l * sin(theta)
  ....:     p.append([x, y])
  sage: g1 = line([p[n], [0, 0]], color='red', thickness=2)
  sage: g1 += line(p, thickness=.4); g1.show(aspect_ratio=1)

Sage example in ./graphique.tex, line 777::

  sage: length = 200; n = var('n')
  sage: u = lambda n: n * sqrt(2)
  sage: z = lambda n: exp(2 * I * pi * u(n)).n()
  sage: vertices = [CC(0, 0)]
  sage: for n in range(1, length):
  ....:     vertices.append(vertices[n - 1] + CC(z(n)))
  sage: line(vertices).show(aspect_ratio=1)

Sage example in ./graphique.tex, line 968::

  sage: x = var('x'); y = function('y')
  sage: DE = x*diff(y(x), x) == 2*y(x) + x^3
  sage: desolve(DE, [y(x),x])
  (_C + x)*x^2
  sage: sol = []
  sage: for i in srange(-2, 2, 0.2):
  ....:     sol.append(desolve(DE, [y(x), x], ics=[1, i]))
  ....:     sol.append(desolve(DE, [y(x), x], ics=[-1, i]))
  sage: g = plot(sol, x, -2, 2)
  sage: y = var('y')
  sage: g += plot_vector_field((x, 2*y+x^3), (x,-2,2), (y,-1,1))
  sage: g.show(ymin=-1, ymax=1)

Sage example in ./graphique.tex, line 1029::

  sage: x = var('x'); y = function('y')
  sage: DE = x*diff(y(x), x) == 2*y(x) + x^3
  sage: g = Graphics()             # creates an empty graph
  sage: for i in srange(-2, 2, 0.2):  # long time
  ....:     g += line(desolve_rk4(DE, y(x), ics=[1, i],\
  ....:                       step=0.05, end_points=[0,2]))
  ....:     g += line(desolve_rk4(DE, y(x), ics=[-1, i],\
  ....:                       step=0.05, end_points=[-2,0]))
  sage: y = var('y')
  sage: g += plot_vector_field((x, 2*y+x^3), (x,-2,2), (y,-1,1))
  sage: g.show(ymin=-1, ymax=1)

Sage example in ./graphique.tex, line 1120::

  sage: import scipy; from scipy import integrate
  sage: f = lambda y, t: - cos(y * t)
  sage: t = srange(0, 5, 0.1); p = Graphics()
  sage: for k in srange(0, 10, 0.15):
  ....:       y = integrate.odeint(f, k, t)
  ....:       p += line(zip(t, flatten(y.tolist())))
  sage: t = srange(0, -5, -0.1); q = Graphics()
  sage: for k in srange(0, 10, 0.15):
  ....:       y = integrate.odeint(f, k, t)
  ....:       q += line(zip(t, flatten(y.tolist())))
  sage: y = var('y')
  sage: v = plot_vector_field((1, -cos(x*y)), (x,-5,5), (y,-2,11))
  sage: g = p + q + v; g.show()

Sage example in ./graphique.tex, line 1229::

  sage: import scipy; from scipy import integrate
  sage: a, b, c, d = 1., 0.1, 1.5, 0.75
  sage: def dX_dt(X, t=0):             # returns the population variation
  ....:     return [a*X[0] - b*X[0]*X[1], -c*X[1] + d*b*X[0]*X[1]]
  sage: t = srange(0, 15, .01)                               # time scale
  sage: X0 = [10, 5]         # initial conditions: 10 rabbits and 5 foxes
  sage: X = integrate.odeint(dX_dt, X0, t)           # numerical solution
  sage: rabbits, foxes =  X.T                # shortcut for X.transpose()
  sage: p = line(zip(t, rabbits), color='red')  # number of rabbits graph
  sage: p += text("Rabbits",(12,37), fontsize=10, color='red')
  sage: p += line(zip(t, foxes), color='blue')           # idem for foxes
  sage: p += text("Foxes",(12,7), fontsize=10, color='blue')
  sage: p.axes_labels(["time", "population"]); p.show(gridlines=True)

Sage example in ./graphique.tex, line 1266::

  sage: n = 11;  L = srange(6, 18, 12 / n); R = srange(3, 9, 6 / n)
  sage: CI = list(zip(L, R))           # list of initial conditions
  sage: def g(x,y):
  ....:     v = vector(dX_dt([x, y]))  # for a nicer graph, we
  ....:     return v/v.norm()          # normalise the vector field
  sage: x, y = var('x, y')
  sage: q = plot_vector_field(g(x, y), (x, 0, 60), (y, 0, 36))
  sage: for j in range(n):
  ....:     X = integrate.odeint(dX_dt, CI[j], t)        # resolution
  ....:     q += line(X, color=hue(.8-float(j)/(1.8*n))) # graph plot
  sage: q.axes_labels(["rabbits","foxes"]); q.show()

Sage example in ./graphique.tex, line 1501::

  sage: x, y, t = var('x, y, t')
  sage: alpha(t) = 1; beta(t) = t / 2; gamma(t) = t + t^3 / 8
  sage: env = solve([alpha(t) * x + beta(t) * y == gamma(t),\
  ....:     diff(alpha(t), t) * x + diff(beta(t), t) * y == \
  ....:     diff(gamma(t), t)], [x,y])

Sage example in ./graphique.tex, line 1541::

  sage: f(x) = x^2 / 4
  sage: p = plot(f, -8, 8, rgbcolor=(0.2,0.2,0.4))  # the parabola
  sage: for u in srange(0, 8, 0.1):      # normals to the parabola
  ....:    p += line([[u, f(u)], [-8*u, f(u) + 18]], thickness=.3)
  ....:    p += line([[-u, f(u)], [8*u, f(u) + 18]], thickness=.3)
  sage: p += parametric_plot((env[0][0].rhs(),env[0][1].rhs()),\
  ....:    (t, -8, 8),color='red')             # draws the evolute
  sage: p.show(xmin=-8, xmax=8, ymin=-1, ymax=12, aspect_ratio=1)

Sage example in ./graphique.tex, line 1604::

  sage: t = var('t'); p = 2
  sage: x(t) = t; y(t) = t^2 / (2 * p); f(t) = [x(t), y(t)]
  sage: df(t) = [x(t).diff(t), y(t).diff(t)]
  sage: d2f(t) = [x(t).diff(t, 2), y(t).diff(t, 2)]
  sage: T(t) = [df(t)[0] / df(t).norm(), df[1](t) / df(t).norm()]
  sage: N(t) = [-df(t)[1] / df(t).norm(), df[0](t) / df(t).norm()]
  sage: R(t) = (df(t).norm())^3 / (df(t)[0]*d2f(t)[1]-df(t)[1]*d2f(t)[0])
  sage: Omega(t) = [f(t)[0] + R(t)*N(t)[0], f(t)[1] + R(t)*N(t)[1]]
  sage: g = parametric_plot(f(t), (t,-8,8), color='green',thickness=2)
  sage: for u in srange(.4, 4, .2):
  ....:     g += line([f(t=u), Omega(t=u)], color='red', alpha = .5)
  ....:     g += circle(Omega(t=u), R(t=u), color='blue')
  sage: g.show(aspect_ratio=1,xmin=-12,xmax=7,ymin=-3,ymax=12)

Sage example in ./graphique.tex, line 1781::

  sage: u, v = var('u, v')
  sage: h = lambda u,v: u^2 + 2*v^2
  sage: plot3d(h, (u,-1,1), (v,-1,1), aspect_ratio=[1,1,1])
  Graphics3d Object

Sage example in ./graphique.tex, line 1833. Sometimes the result
needs to be simplified to obtain a nice short expression::

  sage: f(x, y) = x^2 * y / (x^4 + y^2)
  sage: t, theta = var('t, theta')
  sage: result = limit(f(t * cos(theta), t * sin(theta)) / t, t=0)
  sage: result.full_simplify()
  cos(theta)^2/sin(theta)

Sage example in ./graphique.tex, line 1847::

  sage: solve(f(x,y) == 1/2, y)
  [y == x^2]
  sage: a = var('a'); h = f(x, a*x^2).simplify_rational(); h
  a/(a^2 + 1)

Sage example in ./graphique.tex, line 1861::

  sage: plot(h, a, -4, 4)
  Graphics object consisting of 1 graphics primitive

Sage example in ./graphique.tex, line 1908::

  sage: p = plot3d(f(x,y),(x,-2,2),(y,-2,2),plot_points=[150,150])

Sage example in ./graphique.tex, line 1937::

  sage: for i in range(1,4):
  ....:  p += plot3d(-0.5 + i / 4, (x, -2, 2), (y, -2, 2),\
  ....:               color=hue(i / 10), opacity=.1)

Sage example in ./graphique.tex, line 1956::

  sage: x, y, z = var('x, y, z'); a = 1
  sage: h = lambda x, y, z:(a^2 + x^2 + y^2)^2 - 4*a^2*x^2-z^4
  sage: implicit_plot3d(h, (x,-3,3), (y,-3,3), (z,-2,2),  # long time
  ....:                 plot_points=100)
  Graphics3d Object

Sage example in ./graphique.tex, line 2004::

  sage: line3d([(-10*cos(t)-2*cos(5*t)+15*sin(2*t),\
  ....:          -15*cos(2*t)+10*sin(t)-2*sin(5*t),\
  ....:          10*cos(3*t)) for t in srange(0,6.4,.1)],radius=.5)
  Graphics3d Object
"""



"""
Edit the source code of Sage interactively

AUTHORS:

- Nils Bruin
- William Stein -- touch up for inclusion in Sage.
- Simon King: Make it usable on extension classes that do not have
  a docstring; include this module into the reference manual and
  fix some syntax errors in the doc strings.

This module provides a routine to open the source file of a python
object in an editor of your choice, if the source file can be figured
out.  For files that appear to be from the sage library, the path name
gets modified to the corresponding file in the current branch, i.e.,
the file that gets copied into the library upon 'sage -br'.

The editor to be run, and the way it should be called to open the
requested file at the right line number, can be supplied via a
template. For a limited number of editors, templates are already known
to the system. In those cases it suffices to give the editor name.

In fact, if the environment variable :envvar:`EDITOR` is set to a known editor,
then the system will use that if no template has been set explicitly.
"""
# ****************************************************************************
#       Copyright (C) 2007 Nils Bruin <nbruin@sfu.ca> and
#                          William Stein <wstein@math.ucsd.edu>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************

import sage.misc.sageinspect
import os
import re

from string import Template

# by default we do not have an edit template
edit_template = None

# we can set some defaults, however. Add your own if you like.

template_defaults = {
    'vi': Template('vi -c ${line} ${file}'),
    'vim': Template('vim -c ${line} ${file}'),
    'emacs': Template('emacs ${opts} +${line} ${file}'),
    'nedit-nc': Template('nedit-nc -line ${line} ${file}'),
    'nedit-client': Template('nedit-client -line ${line} ${file}'),
    'ncl': Template('ncl -line ${line} ${file}'),
    'gedit': Template('gedit +${line} ${file} &'),
    'kate': Template('kate -u --line +${line} ${file} &')}


def file_and_line(obj):
    r"""
    Look up source file and line number of ``obj``.

    If the file lies in the Sage library, the path name of the
    corresponding file in the current branch (i.e., the file that gets
    copied into the Sage library upon running 'sage -br').  Note that
    the first line of a file is considered to be 1 rather than 0
    because most editors think that this is the case.

    AUTHORS:

    - Nils Bruin (2007-10-03)
    - Simon King (2011-05): Use :mod:`~sage.misc.sageinspect` to get the file
      and the line.

    EXAMPLES::

        sage: import sage.misc.edit_module as edit_module
        sage: edit_module.file_and_line(sage.cpython)
        ('...sage/cpython/__init__.py', 0)

    The following tests against a bug that was fixed in :issue:`11298`::

        sage: edit_module.file_and_line(x)                                              # needs sage.symbolic
        ('...sage/symbolic/expression...pyx', ...)
    """
    # d = inspect.getdoc(obj)
    # ret = sage.misc.sageinspect._extract_embedded_position(d);
    # if ret is not None:
    #     (_, filename, lineno) = ret
    # else:
    #     filename = inspect.getsourcefile(obj)
    #     _,lineno = inspect.findsource(obj)

    #
    #  for sage files, the registered source file is the result of the
    #  preparsing these files end in ".py" and have "*autogenerated*"
    #  on the second line for those files, we replace the extension by
    #  ".sage" and we subtract 3 from the line number to compensate
    #  for the 3 lines that were prefixed in the preparsing process
    #
    from sage.misc.sageinspect import sage_getfile, sage_getsourcelines
    filename = sage_getfile(obj)
    lineno = sage_getsourcelines(obj)[1] - 1
    if filename.endswith('.py'):
        infile = open(filename)
        infile.readline()
        if infile.readline().find("*autogenerated*") >= 0:
            filename = filename[:-3] + '.sage'
            lineno -= 3

    runpathpattern = '^' + sage.env.SAGE_LIB
    develbranch = sage.env.SAGE_SRC
    filename = re.sub(runpathpattern, develbranch, filename)

    return filename, lineno + 1


def template_fields(template):
    r"""
    Given a String.Template object, returns the fields.

    AUTHOR:

    Nils Bruin (2007-10-22)

    EXAMPLES::

        sage: from sage.misc.edit_module import template_fields
        sage: from string import Template
        sage: t = Template("Template ${one} with ${two} and ${three}")
        sage: sorted(template_fields(t))
        ['one', 'three', 'two']
    """
    dict = {}
    dummy = None
    while not dummy:
        try:
            dummy = template.substitute(dict)
        except KeyError as inst:
            dict[inst.args[0]] = None
    return list(dict)

# The routine set_edit_template should only do some consistency
# checks on template_string It should not do any magic. This routine
# should give the user full control over what is going on.


def set_edit_template(template_string):
    r"""
    Set the default edit template string.

    It should reference ``${file}`` and ``${line}``. This routine normally
    needs to be called prior to using 'edit'. However, if the editor
    set in the shell variable :envvar:`EDITOR` is known, then the system will
    substitute an appropriate template for you. See
    edit_module.template_defaults for the recognised templates.

    AUTHOR:

    Nils Bruin (2007-10-03)

    EXAMPLES::

        sage: from sage.misc.edit_module import set_edit_template
        sage: set_edit_template("echo EDIT ${file}:${line}")
        sage: edit(sage)      # not tested
        EDIT /usr/local/sage/src/sage/__init__.py:1
    """
    global edit_template

    if not isinstance(template_string, Template):
        template_string = Template(template_string)
    fields = set(template_fields(template_string))
    if not (fields <= {'file', 'line'} and ('file' in fields)):
        raise ValueError("Only ${file} and ${line} are allowed as template variables, and ${file} must occur.")
    edit_template = template_string

# The routine set_editor is for convenience and hence is allowed to
# apply magic. Given an editor name and possibly some options, it
# should try to set an editor_template that is as appropriate as
# possible for the situation. If it's necessary to query the
# environment for 'DISPLAY' to figure out if certain editors should
# be run in the background, this is where the magic should go.


def set_editor(editor_name, opts=''):
    r"""
    Set the editor to be used by the edit command by basic editor name.

    Currently, the system only knows appropriate call strings for a
    limited number of editors. If you want to use another editor, you
    should set the whole edit template via :func:`set_edit_template`.

    AUTHOR:

    Nils Bruin (2007-10-05)

    EXAMPLES::

        sage: from sage.misc.edit_module import set_editor
        sage: set_editor('vi')
        sage: sage.misc.edit_module.edit_template.template
        'vi -c ${line} ${file}'
    """
    if editor_name in sage.misc.edit_module.template_defaults:
        set_edit_template(Template(template_defaults[editor_name].safe_substitute(opts=opts)))
    else:
        raise ValueError("editor_name not known. Try set_edit_template(<template_string>) instead.")


def edit(obj, editor=None, bg=None):
    r"""nodetex
    Open source code of ``obj`` in editor of your choice.

    INPUT:

    - editor -- string (default: ``None``); if given, use specified editor.
      Choice is stored for next time.

    AUTHOR:

    Nils Bruin (2007-10-03)

    EXAMPLES:

    This is a typical example of how to use this routine::

        # make some object obj
        sage: edit(obj)    # not tested

    Now for more details and customization::

        sage: import sage.misc.edit_module as m
        sage: m.set_edit_template("vi -c ${line} ${file}")

    In fact, since ``vi`` is a well-known editor, you could also just use::

        sage: m.set_editor("vi")

    To illustrate::

        sage: m.edit_template.template
        'vi -c ${line} ${file}'

    And if your environment variable :envvar:`EDITOR` is set to a recognised
    editor, you would not have to set anything.

    To edit the source of an object, just type something like::

        sage: edit(edit)           # not tested
    """
    global edit_template

    if editor:
        set_editor(editor)
    elif not edit_template:
        try:
            ED = os.environ['EDITOR']
            EDITOR = ED.split()
            base = EDITOR[0]
            opts = ' '.join(EDITOR[1:])   # for future use
            set_editor(base, opts=opts)
        except (ValueError, KeyError, IndexError):
            raise ValueError("Use set_edit_template(<template_string>) to set a default")

    if not edit_template:
        raise ValueError("Use set_edit_template(<template_string>) to set a default")

    filename, lineno = file_and_line(obj)
    cmd = edit_template.substitute(line=lineno, file=filename)

    if bg is True and cmd[-1] != '&':
        cmd = cmd + '&'
    if bg is False and cmd[-1] == '&':
        cmd = cmd[:-1]

    os.system(cmd)


def edit_devel(self, filename, linenum):
    """
    This function is for internal use and is called by IPython when you use
    the IPython commands ``%edit`` or ``%ed``.

    This hook calls the default implementation, but changes the filename for
    files that appear to be from the sage library: if the filename begins with
    'SAGE_LOCAL/lib/python.../site-packages', it replaces this by
    'SAGE_ROOT/src'.

    EXAMPLES::

        sage: %edit gcd         # indirect doctest, not tested
        sage: %ed gcd           # indirect doctest, not tested

    The above should open your favorite editor (as stored in the environment
    variable :envvar:`EDITOR`) with the file in which gcd is defined, and when your
    editor supports it, also at the line in which gcd is defined.
    """
    import IPython.core.hooks
    runpathpattern = '^' + sage.env.SAGE_LIB
    develbranch = sage.env.SAGE_SRC
    filename = re.sub(runpathpattern, develbranch, filename)
    IPython.core.hooks.editor(self, filename, linenum)



"""
This file (./sol/combinat_doctest.sage) was *autogenerated* from ./sol/combinat.tex,
with sagetex.sty version 2011/05/27 v2.3.1.
It contains the contents of all the sageexample environments from this file.
You should be able to doctest this file with:
sage -t ./sol/combinat_doctest.sage
It is always safe to delete this file; it is not used in typesetting your
document.

Sage example in ./sol/combinat.tex, line 15::

  sage: Suits = FiniteEnumeratedSet(
  ....:            ["Hearts", "Diamonds", "Spades", "Clubs"])
  sage: Values  = FiniteEnumeratedSet([2, 3, 4, 5, 6, 7, 8, 9, 10,
  ....:                                "Jack", "Queen", "King", "Ace"])
  sage: FourOfaKind = cartesian_product([Arrangements(Values,2), Suits])

Sage example in ./sol/combinat.tex, line 34::

  sage: FourOfaKind.list()
  [([2, 3], 'Hearts'),
   ([2, 3], 'Diamonds'),
  ...
   (['Ace', 'King'], 'Clubs')]

Sage example in ./sol/combinat.tex, line 51::

  sage: FourOfaKind.cardinality()
  624

Sage example in ./sol/combinat.tex, line 66::

  sage: Cards = cartesian_product([Values, Suits])
  sage: Hands = Subsets(Cards, 5)
  sage: FourOfaKind.cardinality() / Hands.cardinality()
  1/4165

Sage example in ./sol/combinat.tex, line 86::

  sage: StraightFlush = cartesian_product([range(1, 11), Suits])
  sage: StraightFlush.cardinality()
  40

Sage example in ./sol/combinat.tex, line 98::

  sage: AllFlush = cartesian_product([Subsets(Values,5),Suits])
  sage: AllFlush.cardinality() - StraightFlush.cardinality()
  5108

Sage example in ./sol/combinat.tex, line 113::

  sage: _ / Hands.cardinality()
  1277/649740
  sage: float(_)
  0.001965401545233478

Sage example in ./sol/combinat.tex, line 153::

  sage: Word(['a','b','b','a','a','b','a']).evaluation_dict()
  {'a': 4, 'b': 3}

Sage example in ./sol/combinat.tex, line 170::

  sage: def is_full_hand(hand):
  ....:     suits = Word([value for (value, suit) in hand])
  ....:     repetitions = sorted(suits.evaluation_dict().values())
  ....:     return repetitions == [2,3]
  sage: is_full_hand({(5, 'Diamonds'), (6, 'Diamonds'), (6, 'Hearts'),
  ....:               (5, 'Spades'), (1, 'Spades')})
  False
  sage: is_full_hand({(3, 'Clubs'), (3, 'Spades'), (3, 'Hearts'),
  ....:               (2, 'Clubs'), (2, 'Spades')})
  True

Sage example in ./sol/combinat.tex, line 200::

  sage: def estimate_proportion(S, predicate, n):
  ....:     count = 0
  ....:     for i in range(n):
  ....:         if predicate(S.random_element()):
  ....:             count += 1
  ....:     return count/n

Sage example in ./sol/combinat.tex, line 212::

  sage: float(estimate_proportion(Hands, is_full_hand, 10000)) # random
  0.0014

Sage example in ./sol/combinat.tex, line 229::

  sage: FullHands = cartesian_product([Arrangements(Values, 2),
  ....:     Subsets(Suits, 3), Subsets(Suits, 2)])

Sage example in ./sol/combinat.tex, line 242::

  sage: [sorted(v) for v in FullHands.first()]
  [[2, 3], ['Diamonds', 'Hearts', 'Spades'], ['Diamonds', 'Hearts']]

Sage example in ./sol/combinat.tex, line 252::

  sage: float(FullHands.cardinality() / Hands.cardinality())
  0.0014405762304921968

Sage example in ./sol/combinat.tex, line 312::

  sage: C = Compositions(5,length=3)
  sage: C.cardinality
  <bound method ..._cardinality_from_iterator ...>

Sage example in ./sol/combinat.tex, line 322::

  sage: IntegerVectors(5,3).list()
  [[5, 0, 0], [4, 1, 0], [4, 0, 1], [3, 2, 0], [3, 1, 1], [3, 0, 2],
   ...
   [0, 4, 1], [0, 3, 2], [0, 2, 3], [0, 1, 4], [0, 0, 5]]

Sage example in ./sol/combinat.tex, line 328::

  sage: OrderedSetPartitions(3).cardinality()
  13
  sage: OrderedSetPartitions(3).list()
  [[{1}, {2}, {3}], [{1}, {3}, {2}], [{2}, {1}, {3}], [{3}, {1}, {2}],
   ...
   [{1, 2}, {3}], [{1, 3}, {2}], [{2, 3}, {1}], [{1, 2, 3}]]
  sage: OrderedSetPartitions(3,2).random_element()    # random
  [{1, 3}, {2}]

Sage example in ./sol/combinat.tex, line 338::

  sage: StandardTableaux([3,2]).cardinality()
  5
  sage: StandardTableaux([3,2]).an_element()
  [[1, 3, 5], [2, 4]]

Sage example in ./sol/combinat.tex, line 350::

  sage: list(AlternatingSignMatrices(1))
  [[1]]
  sage: list(AlternatingSignMatrices(2))
  [
  [1 0]  [0 1]
  [0 1], [1 0]
  ]

Sage example in ./sol/combinat.tex, line 361::

  sage: list(AlternatingSignMatrices(3))
  [
  [1 0 0]  [0 1 0]  [1 0 0]  [ 0  1  0]  [0 0 1]  [0 1 0]  [0 0 1]
  [0 1 0]  [1 0 0]  [0 0 1]  [ 1 -1  1]  [1 0 0]  [0 0 1]  [0 1 0]
  [0 0 1], [0 0 1], [0 1 0], [ 0  1  0], [0 1 0], [1 0 0], [1 0 0]
  ]

Sage example in ./sol/combinat.tex, line 383::

  sage: GF(2)^5
  Vector space of dimension 5 over Finite Field of size 2
  sage: _.cardinality()
  32

Sage example in ./sol/combinat.tex, line 400::

  sage: (2^3-2^0)*(2^3-2^1)*(2^3-2^2)
  168

Sage example in ./sol/combinat.tex, line 406::

  sage: GL(3,2)
  General Linear Group of degree 3 over Finite Field of size 2
  sage: _.cardinality()
  168

Sage example in ./sol/combinat.tex, line 423::

  sage: from sage.combinat.q_analogues import q_factorial
  sage: q = 2; n = 3
  sage: q^(n*(n-1)/2) * (q-1)^n * q_factorial(n,q)
  168
  sage: q = 3; n = 5
  sage: GL(n, q).cardinality()
  475566474240
  sage: q^(n*(n-1)/2) * (q-1)^n * q_factorial(n,q)
  475566474240

Sage example in ./sol/combinat.tex, line 537::

  sage: def C(n):
  ....:     if n == 1:
  ....:         yield BinaryTree()
  ....:     elif n > 1:
  ....:         for k in range(1,n):
  ....:             for t1 in C(k):
  ....:                 for t2 in C(n-k):
  ....:                     yield BinaryTree([t1,t2])

Sage example in ./sol/combinat.tex, line 549::

  sage: list(C(1))
  [.]
  sage: list(C(2))
  [[., .]]
  sage: list(C(3))
  [[., [., .]],
   [[., .], .]]
  sage: list(C(4))
  [[., [., [., .]]],
   [., [[., .], .]],
   [[., .], [., .]],
   [[., [., .]], .],
   [[[., .], .], .]]

Sage example in ./sol/combinat.tex, line 566::

  sage: [len(list(C(n))) for n in range(9)]
  [0, 1, 1, 2, 5, 14, 42, 132, 429]
"""



"""
This file (./sol/calculus_doctest.sage) was *autogenerated* from ./sol/calculus.tex,
with sagetex.sty version 2011/05/27 v2.3.1.
It contains the contents of all the sageexample environments from this file.
You should be able to doctest this file with:
sage -t ./sol/calculus_doctest.sage
It is always safe to delete this file; it is not used in typesetting your
document.

Sage example in ./sol/calculus.tex, line 3::

  sage: reset()

Sage example in ./sol/calculus.tex, line 10::

  sage: n = var('n'); pmax = 4; s = [n + 1]
  sage: for p in [1..pmax]:
  ....:     s += [factor(((n+1)^(p+1) - sum(binomial(p+1, j) * s[j]
  ....:           for j in [0..p-1])) / (p+1))]
  sage: s
  [n + 1,
   1/2*(n + 1)*n,
   1/6*(2*n + 1)*(n + 1)*n,
   1/4*(n + 1)^2*n^2,
   1/30*(3*n^2 + 3*n - 1)*(2*n + 1)*(n + 1)*n]

Sage example in ./sol/calculus.tex, line 42::

  sage: x, h, a = var('x, h, a'); f = function('f')
  sage: g(x) = taylor(f(x), x, a, 3)
  sage: phi(h) = (g(a+3*h) - 3*g(a+2*h) + 3*g(a+h) - g(a)) / h^3; phi(h)
  diff(f(a), a, a, a)

Sage example in ./sol/calculus.tex, line 75::

  sage: n = 7; x, h, a = var('x h a'); f = function('f')
  sage: g(x) = taylor(f(x), x, a, n)
  sage: sum((-1)^(n-k) * binomial(n,k) * g(a+k*h) for k in (0..n)) / h^n
  diff(f(a), a, a, a, a, a, a, a)

Sage example in ./sol/calculus.tex, line 100::

  sage: theta = 12*arctan(1/38) + 20*arctan(1/57) \
  ....:       + 7*arctan(1/239) + 24*arctan(1/268)
  sage: tan(theta).trig_expand().trig_simplify()
  1

Sage example in ./sol/calculus.tex, line 110::

  sage: 12*(1/38) + 20*(1/57) + 7*(1/239) + 24*(1/268)
  37735/48039

Sage example in ./sol/calculus.tex, line 135::

  sage: x = var('x'); f(x) = taylor(arctan(x), x, 0, 21)
  sage: approx = 4 * (12 * f(1/38) + 20 * f(1/57)
  ....:           + 7 * f(1/239) + 24 * f(1/268))
  sage: approx.n(digits = 50); pi.n(digits = 50)
  3.1415926535897932384626433832795028851616168852864
  3.1415926535897932384626433832795028841971693993751
  sage: approx.n(digits = 50) - pi.n(digits = 50)
  9.6444748591132486785420917537404705292978817080880e-37

Sage example in ./sol/calculus.tex, line 182::

  sage: n = var('n'); phi = lambda x: n*pi + pi/2 - arctan(1/x)
  sage: x = n*pi
  sage: for i in range(4):
  ....:    x = taylor(phi(x), n, infinity, 2*i); x
  1/2*pi + pi*n
  1/2*pi + pi*n - 1/(pi*n) + 1/2/(pi*n^2)
  1/2*pi + pi*n - 1/(pi*n) + 1/2/(pi*n^2)
    - 1/12*(3*pi^2 + 8)/(pi^3*n^3) + 1/8*(pi^2 + 8)/(pi^3*n^4)
  1/2*pi + pi*n - 1/(pi*n) + 1/2/(pi*n^2)
    - 1/12*(3*pi^2 + 8)/(pi^3*n^3) + 1/8*(pi^2 + 8)/(pi^3*n^4)
    - 1/240*(15*pi^4 + 240*pi^2 + 208)/(pi^5*n^5)
    + 1/96*(3*pi^4 + 80*pi^2 + 208)/(pi^5*n^6)

Sage example in ./sol/calculus.tex, line 239::

  sage: h = var('h'); f(x, y) = x * y * (x^2 - y^2) / (x^2 + y^2)
  sage: D1f(x, y) = diff(f(x,y), x); limit((D1f(0,h) - 0) / h, h=0)
  -1
  sage: D2f(x, y) = diff(f(x,y), y); limit((D2f(h,0) - 0) / h, h=0)
  1
  sage: g = plot3d(f(x, y), (x, -3, 3), (y, -3, 3))

Sage example in ./sol/calculus.tex, line 285::

  sage: n, t = var('n, t')
  sage: v(n) = (4/(8*n+1)-2/(8*n+4)-1/(8*n+5)-1/(8*n+6))*1/16^n
  sage: assume(8*n+1>0)
  sage: f(t) = 4*sqrt(2)-8*t^3-4*sqrt(2)*t^4-8*t^5
  sage: u(n) = integrate(f(t) * t^(8*n), t, 0, 1/sqrt(2))
  sage: (u(n)-v(n)).canonicalize_radical()
  0

Sage example in ./sol/calculus.tex, line 317::

  sage: t = var('t'); J = integrate(f(t) / (1-t^8), t, 0, 1/sqrt(2))
  sage: J.canonicalize_radical()
  pi + 2*log(sqrt(2) + 1) + 2*log(sqrt(2) - 1)

Sage example in ./sol/calculus.tex, line 325::

  sage: J.simplify_log().canonicalize_radical()
  pi

Sage example in ./sol/calculus.tex, line 337::

  sage: l = sum(v(n) for n in (0..40)); l.n(digits=60)
  3.14159265358979323846264338327950288419716939937510581474759
  sage: pi.n(digits=60)
  3.14159265358979323846264338327950288419716939937510582097494
  sage: print("%e" % (l-pi).n(digits=60))
  -6.227358e-54

Sage example in ./sol/calculus.tex, line 369::

  sage: x = var('x'); ps = lambda f, g : integral(f * g, x, -pi, pi)
  sage: n = 5; a = var('a0, a1, a2, a3, a4, a5')
  sage: P = sum(a[k] * x^k for k in (0..n))
  sage: equ = [ps(P - sin(x), x^k) for k in (0..n)]
  sage: sol = solve(equ, a)
  sage: P = sum(sol[0][k].rhs() * x^k for k in (0..n)); P
  105/8*(pi^4 - 153*pi^2 + 1485)*x/pi^6 - 315/4*(pi^4 - 125*pi^2 +
  1155)*x^3/pi^8 + 693/8*(pi^4 - 105*pi^2 + 945)*x^5/pi^10
  sage: g = plot(P,x,-6,6,color='red') + plot(sin(x),x,-6,6,color='blue')
  sage: g.show(ymin = -1.5, ymax = 1.5)

Sage example in ./sol/calculus.tex, line 430::

  sage: p, e = var('p, e')
  sage: theta1, theta2, theta3 = var('theta1, theta2, theta3')
  sage: r(theta) = p / (1 - e * cos(theta))
  sage: r1 = r(theta1); r2 = r(theta2); r3 = r(theta3)
  sage: R1 = vector([r1 * cos(theta1), r1 * sin(theta1), 0])
  sage: R2 = vector([r2 * cos(theta2), r2 * sin(theta2), 0])
  sage: R3 = vector([r3 * cos(theta3), r3 * sin(theta3), 0])

Sage example in ./sol/calculus.tex, line 446::

  sage: D = R1.cross_product(R2)+R2.cross_product(R3)+R3.cross_product(R1)
  sage: S = (r1 - r3) * R2 + (r3 - r2) * R1 + (r2 - r1) * R3
  sage: i = vector([1, 0, 0]); V =  S + e * i.cross_product(D)
  sage: V.simplify_full()
  (0, 0, 0)

Sage example in ./sol/calculus.tex, line 466::

  sage: S.cross_product(D).simplify_full()[1:3]
  (0, 0)

Sage example in ./sol/calculus.tex, line 479::

  sage: N = r3 * R1.cross_product(R2) + r1 * R2.cross_product(R3)\
  ....:   + r2 * R3.cross_product(R1)
  sage: W = p * S + e * i.cross_product(N)
  sage: W.simplify_full()
  (0, 0, 0)

Sage example in ./sol/calculus.tex, line 504::

  sage: R1=vector([0,1,0]); R2=vector([2,2,0]); R3=vector([3.5,0,0])
  sage: r1 = R1.norm(); r2 = R2.norm(); r3 = R3.norm()
  sage: D = R1.cross_product(R2) + R2.cross_product(R3) \
  ....:   + R3.cross_product(R1)
  sage: S = (r1 - r3) * R2 + (r3 - r2) * R1 + (r2 - r1) * R3
  sage: N = r3 * R1.cross_product(R2) + r1 * R2.cross_product(R3) \
  ....:   + r2 * R3.cross_product(R1)
  sage: e = S.norm() / D.norm(); p = N.norm() / D.norm()
  sage: a = p/(1-e^2); c = a * e; b = sqrt(a^2 - c^2)
  sage: X = S.cross_product(D); i = X / X.norm()
  sage: phi = atan2(i[1], i[0]) * 180 / pi.n()
  sage: print("%.3f %.3f %.3f %.3f %.3f %.3f" % (a, b, c, e, p, phi))
  2.360 1.326 1.952 0.827 0.746 17.917

Sage example in ./sol/calculus.tex, line 544::

  sage: A = matrix(QQ, [[ 2,  -3,   2, -12, 33],
  ....:                 [ 6,   1,  26, -16, 69],
  ....:                 [10, -29, -18, -53, 32],
  ....:                 [ 2,   0,   8, -18, 84]])
  sage: A.right_kernel()
  Vector space of degree 5 and dimension 2 over Rational Field
  Basis matrix:
  [     1      0  -7/34   5/17   1/17]
  [     0      1  -3/34 -10/17  -2/17]

Sage example in ./sol/calculus.tex, line 571::

  sage: H = A.echelon_form(); H
  [ 1  0  4  0 -3]
  [ 0  1  2  0  7]
  [ 0  0  0  1 -5]
  [ 0  0  0  0  0]

Sage example in ./sol/calculus.tex, line 608::

  sage: A.column_space()
  Vector space of degree 4 and dimension 3 over Rational Field
  Basis matrix:
  [       1        0        0 1139/350]
  [       0        1        0    -9/50]
  [       0        0        1   -12/35]

Sage example in ./sol/calculus.tex, line 624::

  sage: S.<x, y, z, t> = QQ[]
  sage: C = matrix(S, 4, 1, [x, y, z, t])
  sage: B = block_matrix([A, C], ncols=2)
  sage: C = B.echelon_form()
  sage: C[3,5]*350
  -1139*x + 63*y + 120*z + 350*t

Sage example in ./sol/calculus.tex, line 643::

  sage: K = A.left_kernel(); K
  Vector space of degree 4 and dimension 1 over Rational Field
  Basis matrix:
  [        1  -63/1139 -120/1139 -350/1139]

Sage example in ./sol/calculus.tex, line 653::

  sage: matrix(K.0).right_kernel()
  Vector space of degree 4 and dimension 3 over Rational Field
  Basis matrix:
  [       1        0        0 1139/350]
  [       0        1        0    -9/50]
  [       0        0        1   -12/35]

Sage example in ./sol/calculus.tex, line 668::

  sage: A = matrix(QQ, [[-2, 1, 1], [8, 1, -5], [4, 3, -3]])
  sage: C = matrix(QQ, [[1, 2, -1], [2, -1, -1], [-5, 0, 3]])

Sage example in ./sol/calculus.tex, line 680::

  sage: B = C.solve_left(A); B
  [ 0 -1  0]
  [ 2  3  0]
  [ 2  1  0]

Sage example in ./sol/calculus.tex, line 691::

  sage: C.left_kernel()
  Vector space of degree 3 and dimension 1 over Rational Field
  Basis matrix:
  [1 2 1]

Sage example in ./sol/calculus.tex, line 699::

  sage: x, y, z = var('x, y, z'); v = matrix([[1, 2, 1]])
  sage: B = B + (x*v).stack(y*v).stack(z*v); B
  [      x 2*x - 1       x]
  [  y + 2 2*y + 3       y]
  [  z + 2 2*z + 1       z]

Sage example in ./sol/calculus.tex, line 708::

  sage: A == B*C
  True
"""



"""
Pager for showing strings

Currently we just use the IPython pager.
If we want to use something else, we can just change this function.

Any code in sage that uses a pager should use this pager.
"""
# ---------------------------------------------------------------------------
#       Copyright (C) 2006 William Stein <wstein@gmail.com>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#
#                  https://www.gnu.org/licenses/
# ---------------------------------------------------------------------------


def pager():
    import IPython.core.page
    return IPython.core.page.page



# sage_setup: distribution = sagemath-repl
# sage.doctest: needs sage.libs.flint sage.modular
"""
This file contains a bunch of tests extracted from the published book
'Modular Forms: a Computational Approach' by William Stein, AMS 2007.

TESTS::

    sage: # needs sage.libs.gap
    sage: G = SL(2,ZZ); G
    Special Linear Group of degree 2 over Integer Ring
    sage: S, T = G.gens()
    sage: S
    [ 0  1]
    [-1  0]
    sage: T
    [1 1]
    [0 1]

    sage: delta_qexp(6)
    q - 24*q^2 + 252*q^3 - 1472*q^4 + 4830*q^5 + O(q^6)
    sage: bernoulli(12)
    -691/2730
    sage: bernoulli(50)
    495057205241079648212477525/66
    sage: len(str(bernoulli(10000)))
    27706
    sage: E4 = eisenstein_series_qexp(4, 3)
    sage: E6 = eisenstein_series_qexp(6, 3)
    sage: E4^6
    1/191102976000000 + 1/132710400000*q + 203/44236800000*q^2 + O(q^3)
    sage: E4^3*E6^2
    1/3511517184000 - 1/12192768000*q - 377/4064256000*q^2 + O(q^3)
    sage: E6^4
    1/64524128256 - 1/32006016*q + 241/10668672*q^2 + O(q^3)
    sage: victor_miller_basis(28,5)
    [
    1 + 15590400*q^3 + 36957286800*q^4 + O(q^5),
    q + 151740*q^3 + 61032448*q^4 + O(q^5),
    q^2 + 192*q^3 - 8280*q^4 + O(q^5)
    ]
    sage: R.<q> = QQ[['q']]
    sage: F4 =  240 * eisenstein_series_qexp(4,3)
    sage: F6 = -504 * eisenstein_series_qexp(6,3)
    sage: F4^3
    1 + 720*q + 179280*q^2 + O(q^3)
    sage: Delta = (F4^3 - F6^2)/1728; Delta
    q - 24*q^2 + O(q^3)
    sage: F4^3 - 720*Delta
    1 + 196560*q^2 + O(q^3)
    sage: M = ModularForms(1,36, prec=6).echelon_form()
    sage: M.basis()
    [
    1 + 6218175600*q^4 + 15281788354560*q^5 + O(q^6),
    q + 57093088*q^4 + 37927345230*q^5 + O(q^6),
    q^2 + 194184*q^4 + 7442432*q^5 + O(q^6),
    q^3 - 72*q^4 + 2484*q^5 + O(q^6)
    ]
    sage: T2 = M.hecke_matrix(2); T2
    [     34359738369                0       6218175600 9026867482214400]
    [               0                0      34416831456    5681332472832]
    [               0                1           194184       -197264484]
    [               0                0              -72           -54528]
    sage: T2.charpoly().factor()
    (x - 34359738369) * (x^3 - 139656*x^2 - 59208339456*x - 1467625047588864)
    sage: bernoulli_mod_p(23)
    [1, 4, 13, 17, 13, 6, 10, 5, 10, 9, 15]
    sage: set_modsym_print_mode ('modular')
    sage: M = ModularSymbols(11, 2)
    sage: M.basis()
    ({Infinity, 0}, {-1/8, 0}, {-1/9, 0})
    sage: S = M.cuspidal_submodule()
    sage: S.integral_basis()     # basis over ZZ.
    ({-1/8, 0}, {-1/9, 0})
    sage: set_modsym_print_mode ('manin')    # set it back
    sage: continued_fraction(4/7).convergents()
    [0, 1, 1/2, 4/7]
    sage: M = ModularSymbols(2,2)
    sage: M
    Modular Symbols space of dimension 1 for Gamma_0(2)
    of weight 2 with sign 0 over Rational Field
    sage: M.manin_generators()
    [(0,1), (1,0), (1,1)]

    sage: M = ModularSymbols(3,2)
    sage: M.manin_generators()
    [(0,1), (1,0), (1,1), (1,2)]

    sage: M = ModularSymbols(6,2)
    sage: M.manin_generators()
    [(0,1), (1,0), (1,1), (1,2), (1,3), (1,4), (1,5), (2,1),
     (2,3), (2,5), (3,1), (3,2)]
    sage: M = ModularSymbols(2,2)
    sage: [x.lift_to_sl2z(2) for x in M.manin_generators()]
    [[1, 0, 0, 1], [0, -1, 1, 0], [1, 0, 1, 1]]
    sage: M = ModularSymbols(6,2)
    sage: x = M.manin_generators()[9]
    sage: x
    (2,5)
    sage: x.lift_to_sl2z(6)
    [1, 2, 2, 5]
    sage: M = ModularSymbols(2,2)
    sage: M.manin_basis()
    [1]
    sage: [M.manin_generators()[i] for i in M.manin_basis()]
    [(1,0)]
    sage: M = ModularSymbols(6,2)
    sage: M.manin_basis()
    [1, 10, 11]
    sage: [M.manin_generators()[i] for i in M.manin_basis()]
    [(1,0), (3,1), (3,2)]
    sage: M.basis()
    ((1,0), (3,1), (3,2))
    sage: [x.modular_symbol_rep() for x in M.basis()]
    [{Infinity, 0}, {0, 1/3}, {-1/2, -1/3}]
    sage: M = ModularSymbols(2,2)
    sage: M.manin_gens_to_basis()
    [-1]
    [ 1]
    [ 0]
    sage: M = ModularSymbols(2,2)
    sage: x = (1,0); M(x)
    (1,0)
    sage: M( (3,1) )    # entries are reduced modulo 2 first
    0
    sage: M( (10,19) )
    -(1,0)
    sage: M = ModularSymbols(6,2)
    sage: M.manin_gens_to_basis()
    [-1  0  0]
    [ 1  0  0]
    [ 0  0  0]
    [ 0 -1  1]
    [ 0 -1  0]
    [ 0 -1  1]
    [ 0  0  0]
    [ 0  1 -1]
    [ 0  0 -1]
    [ 0  1 -1]
    [ 0  1  0]
    [ 0  0  1]
    sage: M = ModularSymbols(6,2)
    sage: M((0,1))
    -(1,0)
    sage: M((1,2))
    -(3,1) + (3,2)
    sage: HeilbronnCremona(2).to_list()
    [[1, 0, 0, 2], [2, 0, 0, 1], [2, 1, 0, 1], [1, 0, 1, 2]]
    sage: HeilbronnCremona(3).to_list()
    [[1, 0, 0, 3], [3, 1, 0, 1], [1, 0, 1, 3], [3, 0, 0, 1],
     [3, -1, 0, 1], [-1, 0, 1, -3]]
    sage: HeilbronnCremona(5).to_list()
    [[1, 0, 0, 5], [5, 2, 0, 1], [2, 1, 1, 3], [1, 0, 3, 5],
     [5, 1, 0, 1], [1, 0, 1, 5], [5, 0, 0, 1], [5, -1, 0, 1],
     [-1, 0, 1, -5], [5, -2, 0, 1], [-2, 1, 1, -3],
     [1, 0, -3, 5]]
    sage: len(HeilbronnCremona(37))
    128
    sage: len(HeilbronnCremona(389))
    1892
    sage: len(HeilbronnCremona(2003))
    11662
    sage: M = ModularSymbols(2,2)
    sage: M.T(2).matrix()
    [1]
    sage: M = ModularSymbols(6, 2)
    sage: M.T(2).matrix()
    [ 2  1 -1]
    [-1  0  1]
    [-1 -1  2]
    sage: M.T(3).matrix()
    [3 2 0]
    [0 1 0]
    [2 2 1]
    sage: M.T(3).fcp()  # factored characteristic polynomial
    (x - 3) * (x - 1)^2
    sage: M = ModularSymbols(39, 2)
    sage: T2 = M.T(2)
    sage: T2.matrix()
    [ 3  0 -1  0  0  1  1 -1  0]
    [ 0  0  2  0 -1  1  0  1 -1]
    [ 0  1  0 -1  1  1  0  1 -1]
    [ 0  0  1  0  0  1  0  1 -1]
    [ 0 -1  2  0  0  1  0  1 -1]
    [ 0  0  1  1  0  1  1 -1  0]
    [ 0  0  0 -1  0  1  1  2  0]
    [ 0  0  0  1  0  0  2  0  1]
    [ 0  0 -1  0  0  0  1  0  2]
    sage: T2.fcp()     # factored characteristic polynomial
    (x - 1)^2 * (x - 3)^3 * (x^2 + 2*x - 1)^2
    sage: T2 = M.T(2).matrix()
    sage: T5 = M.T(5).matrix()
    sage: T2*T5 - T5*T2 == 0
    True
    sage: T5.charpoly().factor()
    (x - 2)^2 * (x - 6)^3 * (x^2 - 8)^2
    sage: M = ModularSymbols(39, 2)
    sage: M.T(2).decomposition()
    [
    Modular Symbols subspace of dimension 2 of Modular Symbols space of dimension 9 for Gamma_0(39) of weight 2 with sign 0 over Rational Field,
    Modular Symbols subspace of dimension 3 of Modular Symbols space of dimension 9 for Gamma_0(39) of weight 2 with sign 0 over Rational Field,
    Modular Symbols subspace of dimension 4 of Modular Symbols space of dimension 9 for Gamma_0(39) of weight 2 with sign 0 over Rational Field
    ]
    sage: M = ModularSymbols(2, 2)
    sage: M.boundary_map()
    Hecke module morphism boundary map defined by the matrix
    [ 1 -1]
    Domain: Modular Symbols space of dimension 1 for
    Gamma_0(2) of weight ...
    Codomain: Space of Boundary Modular Symbols for
    Congruence Subgroup Gamma0(2) ...
    sage: M.cuspidal_submodule()
    Modular Symbols subspace of dimension 0 of Modular
    Symbols space of dimension 1 for Gamma_0(2) of weight
    2 with sign 0 over Rational Field
    sage: M = ModularSymbols(11, 2)
    sage: M.boundary_map().matrix()
    [ 1 -1]
    [ 0  0]
    [ 0  0]
    sage: M.cuspidal_submodule()
    Modular Symbols subspace of dimension 2 of Modular Symbols space of dimension 3 for Gamma_0(11) of weight 2 with sign 0 over Rational Field
    sage: S = M.cuspidal_submodule(); S
    Modular Symbols subspace of dimension 2 of Modular Symbols space of dimension 3 for Gamma_0(11) of weight 2 with sign 0 over Rational Field
    sage: S.basis()
    ((1,8), (1,9))
    sage: S.T(2).matrix()
    [-2  0]
    [ 0 -2]
    sage: S.T(3).matrix()
    [-1  0]
    [ 0 -1]
    sage: S.T(5).matrix()
    [1 0]
    [0 1]
    sage: E = EllipticCurve([0,-1,1,-10,-20])
    sage: 2 + 1 - E.Np(2)
    -2
    sage: 3 + 1 - E.Np(3)
    -1
    sage: 5 + 1 - E.Np(5)
    1
    sage: 7 + 1 - E.Np(7)
    -2
    sage: [S.T(p).matrix()[0,0] for p in [2,3,5,7]]
    [-2, -1, 1, -2]
    sage: M = ModularSymbols(11); M.basis()
    ((1,0), (1,8), (1,9))
    sage: S = M.cuspidal_submodule(); S
    Modular Symbols subspace of dimension 2 of Modular Symbols space of dimension 3 for Gamma_0(11) of weight 2 with sign 0 over Rational Field
    sage: S.T(2).matrix()
    [-2  0]
    [ 0 -2]
    sage: S.T(3).matrix()
    [-1  0]
    [ 0 -1]
    sage: M = ModularSymbols(33)
    sage: S = M.cuspidal_submodule(); S
    Modular Symbols subspace of dimension 6 of Modular
    Symbols space of dimension 9 for Gamma_0(33) of weight
    2 with sign 0 over Rational Field
    sage: R.<q> = PowerSeriesRing(QQ)
    sage: v = [S.T(n).matrix()[0,0] for n in range(1,6)]
    sage: f00 = sum(v[n-1]*q^n for n in range(1,6)) + O(q^6)
    sage: f00
    q - q^2 - q^3 + q^4 + O(q^6)
    sage: v = [S.T(n).matrix()[0,1] for n in range(1,6)]
    sage: f01 = sum(v[n-1]*q^n for n in range(1,6)) + O(q^6)
    sage: f01
    -2*q^3 + O(q^6)
    sage: v = [S.T(n).matrix()[1,0] for n in range(1,6)]
    sage: f10 = sum(v[n-1]*q^n for n in range(1,6)) + O(q^6)
    sage: f10
    q^3 + O(q^6)
    sage: v = [S.T(n).matrix()[1,1] for n in range(1,6)]
    sage: f11 = sum(v[n-1]*q^n for n in range(1,6)) + O(q^6)
    sage: f11
    q - 2*q^2 + 2*q^4 + q^5 + O(q^6)
    sage: M = ModularSymbols(23)
    sage: S = M.cuspidal_submodule()
    sage: S
    Modular Symbols subspace of dimension 4 of Modular
    Symbols space of dimension 5 for Gamma_0(23) of weight
    2 with sign 0 over Rational Field
    sage: f = S.q_expansion_cuspforms(6)
    sage: f(0,0)
    q - 2/3*q^2 + 1/3*q^3 - 1/3*q^4 - 4/3*q^5 + O(q^6)
    sage: f(0,1)
    O(q^6)
    sage: f(1,0)
    -1/3*q^2 + 2/3*q^3 + 1/3*q^4 - 2/3*q^5 + O(q^6)
    sage: S.q_expansion_basis(6)
    [
    q - q^3 - q^4 + O(q^6),
    q^2 - 2*q^3 - q^4 + 2*q^5 + O(q^6)
    ]
    sage: R = Integers(49)
    sage: R
    Ring of integers modulo 49
    sage: R.unit_gens()
    (3,)
    sage: Integers(25).unit_gens()
    (2,)
    sage: Integers(100).unit_gens()
    (51, 77)
    sage: Integers(200).unit_gens()
    (151, 101, 177)
    sage: Integers(2005).unit_gens()
    (402, 1206)
    sage: Integers(200000000).unit_gens()
    (174218751, 51562501, 187109377)
    sage: list(DirichletGroup(5))
    [Dirichlet character modulo 5 of conductor 1 mapping 2 |--> 1,
    Dirichlet character modulo 5 of conductor 5 mapping 2 |--> zeta4,
    Dirichlet character modulo 5 of conductor 5 mapping 2 |--> -1,
    Dirichlet character modulo 5 of conductor 5 mapping 2 |--> -zeta4]
    sage: list(DirichletGroup(5, QQ))
    [Dirichlet character modulo 5 of conductor 1 mapping 2 |--> 1, Dirichlet character modulo 5 of conductor 5 mapping 2 |--> -1]
    sage: G = DirichletGroup(200)
    sage: G
    Group of Dirichlet characters modulo 200 with values in Cyclotomic Field of order 20 and degree 8
    sage: G.exponent()
    20
    sage: G.gens()
    (Dirichlet character modulo 200 of conductor 4 mapping 151 |--> -1, 101 |--> 1, 177 |--> 1,
    Dirichlet character modulo 200 of conductor 8 mapping 151 |--> 1, 101 |--> -1, 177 |--> 1,
    Dirichlet character modulo 200 of conductor 25 mapping 151 |--> 1, 101 |--> 1, 177 |--> zeta20)
    sage: K = G.base_ring()
    sage: zeta = K.0
    sage: eps = G([1,-1,zeta^5])
    sage: eps
    Dirichlet character modulo 200 of conductor 40 mapping 151 |--> 1, 101 |--> -1, 177 |--> zeta20^5
    sage: eps(3)
    zeta20^5
    sage: -zeta^15
    zeta20^5
    sage: kronecker(151,200)
    1
    sage: kronecker(101,200)
    -1
    sage: kronecker(177,200)
    1
    sage: G = DirichletGroup(20)
    sage: G.galois_orbits()
    [
    [Dirichlet character modulo 20 of conductor 20 mapping 11 |--> -1, 17 |--> -1],
    [Dirichlet character modulo 20 of conductor 20 mapping 11 |--> -1, 17 |--> -zeta4,
     Dirichlet character modulo 20 of conductor 20 mapping 11 |--> -1, 17 |--> zeta4],
    [Dirichlet character modulo 20 of conductor 4 mapping 11 |--> -1, 17 |--> 1],
    [Dirichlet character modulo 20 of conductor 5 mapping 11 |--> 1, 17 |--> -1],
    [Dirichlet character modulo 20 of conductor 5 mapping 11 |--> 1, 17 |--> -zeta4,
     Dirichlet character modulo 20 of conductor 5 mapping 11 |--> 1, 17 |--> zeta4],
    [Dirichlet character modulo 20 of conductor 1 mapping 11 |--> 1, 17 |--> 1]
    ]
    sage: G = DirichletGroup(11, QQ); G
    Group of Dirichlet characters modulo 11 with values in Rational Field
    sage: list(G)
    [Dirichlet character modulo 11 of conductor 1 mapping 2 |--> 1,
    Dirichlet character modulo 11 of conductor 11 mapping 2 |--> -1]
    sage: eps = G.0      # 0th generator for Dirichlet group
    sage: eps
    Dirichlet character modulo 11 of conductor 11 mapping 2 |--> -1
    sage: G.unit_gens()
    (2,)
    sage: eps(2)
    -1
    sage: eps(3)
    1
    sage: [eps(11*n) for n in range(10)]
    [0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
    sage: R = CyclotomicField(4)
    sage: CyclotomicField(4)
    Cyclotomic Field of order 4 and degree 2
    sage: G = DirichletGroup(15, R)
    sage: G
    Group of Dirichlet characters modulo 15 with values in Cyclotomic Field of order 4 and degree 2
    sage: list(G)
    [Dirichlet character modulo 15 of conductor 1 mapping 11 |--> 1, 7 |--> 1,
    Dirichlet character modulo 15 of conductor 3 mapping 11 |--> -1, 7 |--> 1,
    Dirichlet character modulo 15 of conductor 5 mapping 11 |--> 1, 7 |--> zeta4,
    Dirichlet character modulo 15 of conductor 15 mapping 11 |--> -1, 7 |--> zeta4,
    Dirichlet character modulo 15 of conductor 5 mapping 11 |--> 1, 7 |--> -1,
    Dirichlet character modulo 15 of conductor 15 mapping 11 |--> -1, 7 |--> -1,
    Dirichlet character modulo 15 of conductor 5 mapping 11 |--> 1, 7 |--> -zeta4,
    Dirichlet character modulo 15 of conductor 15 mapping 11 |--> -1, 7 |--> -zeta4]
    sage: e = G.1
    sage: e(4)
    -1
    sage: e(-1)
    -1
    sage: e(5)
    0
    sage: [e(n) for n in range(15)]
    [0, 1, zeta4, 0, -1, 0, 0, zeta4, -zeta4,
        0, 0, 1, 0, -zeta4, -1]
    sage: G = DirichletGroup(15, GF(5)); G
    Group of Dirichlet characters modulo 15 with values in Finite Field of size 5
    sage: list(G)
    [Dirichlet character modulo 15 of conductor 1 mapping 11 |--> 1, 7 |--> 1,
    Dirichlet character modulo 15 of conductor 3 mapping 11 |--> 4, 7 |--> 1,
    Dirichlet character modulo 15 of conductor 5 mapping 11 |--> 1, 7 |--> 2,
    Dirichlet character modulo 15 of conductor 15 mapping 11 |--> 4, 7 |--> 2,
    Dirichlet character modulo 15 of conductor 5 mapping 11 |--> 1, 7 |--> 4,
    Dirichlet character modulo 15 of conductor 15 mapping 11 |--> 4, 7 |--> 4,
    Dirichlet character modulo 15 of conductor 5 mapping 11 |--> 1, 7 |--> 3,
    Dirichlet character modulo 15 of conductor 15 mapping 11 |--> 4, 7 |--> 3]
    sage: e = G.1
    sage: e(-1)
    4
    sage: e(2)
    2
    sage: e(5)
    0
    sage: [e(n) for n in range(15)]
    [0, 1, 2, 0, 4, 0, 0, 2, 3, 0, 0, 1, 0, 3, 4]
    sage: G = DirichletGroup(5)
    sage: e = G.0
    sage: e(2)
    zeta4
    sage: e.bernoulli(1)
    -1/5*zeta4 - 3/5
    sage: e.bernoulli(9)
    -108846/5*zeta4 - 176868/5
    sage: E = EisensteinForms(Gamma1(13),2)
    sage: E.eisenstein_series()
    [
    1/2 + q + 3*q^2 + 4*q^3 + 7*q^4 + 6*q^5 + O(q^6),
    -7/13*zeta6 - 11/13 + q + (2*zeta6 + 1)*q^2 + (-3*zeta6 + 1)*q^3 + (6*zeta6 - 3)*q^4 - 4*q^5 + O(q^6),
    q + (zeta6 + 2)*q^2 + (-zeta6 + 3)*q^3 + (3*zeta6 + 3)*q^4 + 4*q^5 + O(q^6),
    -zeta6 + q + (2*zeta6 - 1)*q^2 + (3*zeta6 - 2)*q^3 + (-2*zeta6 - 1)*q^4 + 6*q^5 + O(q^6),
    q + (zeta6 + 1)*q^2 + (zeta6 + 2)*q^3 + (zeta6 + 2)*q^4 + 6*q^5 + O(q^6),
    -1 + q - q^2 + 4*q^3 + 3*q^4 - 4*q^5 + O(q^6),
    q + q^2 + 4*q^3 + 3*q^4 + 4*q^5 + O(q^6),
    zeta6 - 1 + q + (-2*zeta6 + 1)*q^2 + (-3*zeta6 + 1)*q^3 + (2*zeta6 - 3)*q^4 + 6*q^5 + O(q^6),
    q + (-zeta6 + 2)*q^2 + (-zeta6 + 3)*q^3 + (-zeta6 + 3)*q^4 + 6*q^5 + O(q^6),
    7/13*zeta6 - 18/13 + q + (-2*zeta6 + 3)*q^2 + (3*zeta6 - 2)*q^3 + (-6*zeta6 + 3)*q^4 - 4*q^5 + O(q^6),
    q + (-zeta6 + 3)*q^2 + (zeta6 + 2)*q^3 + (-3*zeta6 + 6)*q^4 + 4*q^5 + O(q^6)
    ]
    sage: e = E.eisenstein_series()
    sage: for e in E.eisenstein_series():
    ....:     print(e.parameters())
    (Dirichlet character modulo 13 of conductor 1 mapping 2 |--> 1, Dirichlet character modulo 13 of conductor 1 mapping 2 |--> 1, 13)
    (Dirichlet character modulo 13 of conductor 1 mapping 2 |--> 1, Dirichlet character modulo 13 of conductor 13 mapping 2 |--> zeta6, 1)
    (Dirichlet character modulo 13 of conductor 13 mapping 2 |--> zeta6, Dirichlet character modulo 13 of conductor 1 mapping 2 |--> 1, 1)
    (Dirichlet character modulo 13 of conductor 1 mapping 2 |--> 1, Dirichlet character modulo 13 of conductor 13 mapping 2 |--> zeta6 - 1, 1)
    (Dirichlet character modulo 13 of conductor 13 mapping 2 |--> zeta6 - 1, Dirichlet character modulo 13 of conductor 1 mapping 2 |--> 1, 1)
    (Dirichlet character modulo 13 of conductor 1 mapping 2 |--> 1, Dirichlet character modulo 13 of conductor 13 mapping 2 |--> -1, 1)
    (Dirichlet character modulo 13 of conductor 13 mapping 2 |--> -1, Dirichlet character modulo 13 of conductor 1 mapping 2 |--> 1, 1)
    (Dirichlet character modulo 13 of conductor 1 mapping 2 |--> 1, Dirichlet character modulo 13 of conductor 13 mapping 2 |--> -zeta6, 1)
    (Dirichlet character modulo 13 of conductor 13 mapping 2 |--> -zeta6, Dirichlet character modulo 13 of conductor 1 mapping 2 |--> 1, 1)
    (Dirichlet character modulo 13 of conductor 1 mapping 2 |--> 1, Dirichlet character modulo 13 of conductor 13 mapping 2 |--> -zeta6 + 1, 1)
    (Dirichlet character modulo 13 of conductor 13 mapping 2 |--> -zeta6 + 1, Dirichlet character modulo 13 of conductor 1 mapping 2 |--> 1, 1)
    sage: from sage.modular.dims import *
    sage: dimension_cusp_forms(Gamma0(2007),2)
    221
    sage: dimension_eis(Gamma0(2007),2)
    7
    sage: dimension_modular_forms(Gamma0(2007),2)
    228
    sage: dimension_new_cusp_forms(Gamma0(11),12)
    8
    sage: dimension_cusp_forms(Gamma0(11),12)
    10
    sage: dimension_new_cusp_forms(Gamma0(2007),12)
    1017
    sage: dimension_cusp_forms(Gamma0(2007),12)
    2460
    sage: dimension_cusp_forms(Gamma1(2007),2)
    147409
    sage: dimension_eis(Gamma1(2007),2)
    3551
    sage: dimension_modular_forms(Gamma1(2007),2)
    150960
    sage: G = DirichletGroup(2007)
    sage: e = prod(G.gens(), G(1))
    sage: dimension_cusp_forms(e,2)
    222
    sage: dimension_cusp_forms(e,3)
    0
    sage: dimension_cusp_forms(e,4)
    670
    sage: dimension_cusp_forms(e,24)
    5150
    sage: dimension_eis(e,2)
    4
    sage: dimension_eis(e,3)
    0
    sage: dimension_eis(e,4)
    4
    sage: dimension_eis(e,24)
    4
    sage: G = DirichletGroup(2007, QQ)
    sage: e = prod(G.gens(), G(1))
    sage: dimension_new_cusp_forms(e,2)
    76
    sage: p = 389
    sage: k = GF(p)
    sage: a = k(7/13); a
    210
    sage: a.rational_reconstruction()
    7/13
    sage: M = MatrixSpace(QQ,4,8)
    sage: A = M([[-9,6,7,3,1,0,0,0],[-10,3,8,2,0,1,0,0],[3,-6,2,8,0,0,1,0],[-8,-6,-8,6,0,0,0,1]])
    sage: A41 = MatrixSpace(GF(41),4,8)(A)
    sage: E41 = A41.echelon_form()
    sage: B = A.matrix_from_columns([0,1,2,4])
    sage: E = B^(-1)*A
    sage: B = A.matrix_from_columns([0,1,2,3])
    sage: M = ModularSymbols(Gamma0(6)); M
    Modular Symbols space of dimension 3 for Gamma_0(6)
    of weight 2 with sign 0 over Rational Field
    sage: M.new_subspace()
    Modular Symbols subspace of dimension 1 of Modular Symbols space of dimension 3 for Gamma_0(6) of weight 2 with sign 0 over Rational Field
    sage: M.old_subspace()
    Modular Symbols subspace of dimension 3 of Modular
    Symbols space of dimension 3 for Gamma_0(6) of weight
    2 with sign 0 over Rational Field
    sage: G = DirichletGroup(13)
    sage: G
    Group of Dirichlet characters modulo 13 with values in Cyclotomic Field of order 12 and degree 4
    sage: dimension_modular_forms(Gamma1(13),2)
    13
    sage: [dimension_modular_forms(e,2) for e in G]
    [1, 0, 3, 0, 2, 0, 2, 0, 2, 0, 3, 0]
    sage: G = DirichletGroup(100)
    sage: G
    Group of Dirichlet characters modulo 100 with values in Cyclotomic Field of order 20 and degree 8
    sage: dimension_modular_forms(Gamma1(100),2)
    370
    sage: v = [dimension_modular_forms(e,2) for e in G]; v
    [24, 0, 0, 17, 18, 0, 0, 17, 18, 0, 0, 21, 18, 0, 0, 17,
      18, 0, 0, 17, 24, 0, 0, 17, 18, 0, 0, 17, 18, 0, 0, 21,
      18, 0, 0, 17, 18, 0, 0, 17]
    sage: sum(v)
    370
    sage: S = CuspForms(Gamma0(45), 2, prec=14); S
    Cuspidal subspace of dimension 3 of Modular Forms space
    of dimension 10 for Congruence Subgroup Gamma0(45) of
    weight 2 over Rational Field
    sage: S.basis()
    [
    q - q^4 - q^10 - 2*q^13 + O(q^14),
    q^2 - q^5 - 3*q^8 + 4*q^11 + O(q^14),
    q^3 - q^6 - q^9 - q^12 + O(q^14)
    ]
    sage: S.new_subspace().basis()
    [
    q + q^2 - q^4 - q^5 - 3*q^8 - q^10 + 4*q^11 - 2*q^13 + O(q^14)
    ]
    sage: CuspForms(Gamma0(9),2)
    Cuspidal subspace of dimension 0 of Modular Forms space
    of dimension 3 for Congruence Subgroup Gamma0(9) of
    weight 2 over Rational Field
    sage: CuspForms(Gamma0(15),2, prec=10).basis()
    [
    q - q^2 - q^3 - q^4 + q^5 + q^6 + 3*q^8 + q^9 + O(q^10)
    ]
"""



"""
This file (./sol/integration_doctest.sage) was *autogenerated* from ./sol/integration.tex,
with sagetex.sty version 2011/05/27 v2.3.1.
It contains the contents of all the sageexample environments from this file.
You should be able to doctest this file with:
sage -t ./sol/integration_doctest.sage
It is always safe to delete this file; it is not used in typesetting your
document.

Sage example in ./sol/integration.tex, line 22::

  sage: x = var('x')
  sage: def NCRule(n):
  ....:     P = prod([x - j for j in range(n)])
  ....:     return [integrate(P / (x-i), x, 0, n-1) \
  ....:             / (P/(x-i)).subs(x=i) for i in range(n)]

Sage example in ./sol/integration.tex, line 35::

  sage: def QuadNC(f, a, b, n):
  ....:     W = NCRule(n)
  ....:     ret = 0
  ....:     for i in range(n):
  ....:         ret += f(a + (b-a)/(n-1)*i) * W[i]
  ....:     return (b-a)/(n-1)*ret

Sage example in ./sol/integration.tex, line 49::

  sage: QuadNC(lambda u: 1, 0, 1, 12)
  1
  sage: N(QuadNC(sin, 0, pi, 10))
  1.99999989482634

Sage example in ./sol/integration.tex, line 59::

  sage: numerical_integral(x * log(1+x), 0, 1)
  (0.25, 2.7755575615628914e-15)
  sage: N(QuadNC(lambda x: x * log(1+x), 0, 1, 19)) # abs tol 2e-14
  0.250000000000001
  sage: numerical_integral(sqrt(1-x^2), 0, 1) # abs tol 2e-14
  (0.7853981677264822, 9.042725224536535e-07)
  sage: N(pi/4)
  0.785398163397448
  sage: N(QuadNC(lambda x: sqrt(1-x^2), 0, 1, 20))
  0.784586419900198

Sage example in ./sol/integration.tex, line 74::

  sage: [N(QuadNC(lambda x: x * log(1+x), 0, 1, n) - 1/4) # abs tol 3e-16
  ....:  for n in [2, 8, 16]]
  [0.0965735902799726, 1.17408932933522e-7, 2.13449050101566e-13]
  sage: [N(QuadNC(lambda x: sqrt(1-x^2), 0, 1, n) - pi/4)    # abs tol 1e-13
  ....:  for n in [2, 8, 16]]
  [-0.285398163397448, -0.00524656673640445, -0.00125482109302663]
"""



"""
This file (./sol/graphtheory_doctest.sage) was *autogenerated* from ./sol/graphtheory.tex,
with sagetex.sty version 2011/05/27 v2.3.1.
It contains the contents of all the sageexample environments from this file.
You should be able to doctest this file with:
sage -t ./sol/graphtheory_doctest.sage
It is always safe to delete this file; it is not used in typesetting your
document.

Sage example in ./sol/graphtheory.tex, line 5::

  sage: def circulant(n,d):
  ....:    g = Graph(n)
  ....:    for u in range(n):
  ....:        for c in range(d):
  ....:            g.add_edge(u,(u+c)%n)
  ....:    return g

Sage example in ./sol/graphtheory.tex, line 19::

  sage: def kneser(n,k):
  ....:    g = Graph()
  ....:    g.add_vertices(Subsets(n,k))
  ....:    for u in g:
  ....:        for v in g:
  ....:            if not u & v:
  ....:                g.add_edge(u,v)
  ....:    return g

Sage example in ./sol/graphtheory.tex, line 33::

  sage: def kneser(n,k):
  ....:    g = Graph()
  ....:    sommets = Set(range(n))
  ....:    g.add_vertices(Subsets(sommets,k))
  ....:    for u in g:
  ....:        for v in Subsets(sommets - u,k):
  ....:            g.add_edge(u,v)
  ....:    return g

Sage example in ./sol/graphtheory.tex, line 59::

  sage: g = graphs.PetersenGraph()
  sage: def optimal_order(g):
  ....:    order = []
  ....:    for color_class in sorted(g.coloring()):
  ....:        for v in color_class:
  ....:            order.append(v)
  ....:    return order
  sage: optimal_order(g)
  [0, 2, 6, 1, 3, 5, 9, 4, 7, 8]
"""



"""
This file (./linalg_doctest.sage) was *autogenerated* from ./linalg.tex,
with sagetex.sty version 2011/05/27 v2.3.1.
It contains the contents of all the sageexample environments from this file.
You should be able to doctest this file with:
sage -t ./linalg_doctest.sage
It is always safe to delete this file; it is not used in typesetting your
document.

Sage example in ./linalg.tex, line 168::

  sage: MS = MatrixSpace(ZZ,2,3); MS
  Full MatrixSpace of 2 by 3 dense matrices over Integer Ring
  sage: VS = VectorSpace(GF(3^2,'x'),3); VS
  Vector space of dimension 3 over Finite Field in x of size 3^2

Sage example in ./linalg.tex, line 184::

  sage: B = MatrixSpace(ZZ,2,3).basis()
  sage: list(B)
  [
  [1 0 0]  [0 1 0]  [0 0 1]  [0 0 0]  [0 0 0]  [0 0 0]
  [0 0 0], [0 0 0], [0 0 0], [1 0 0], [0 1 0], [0 0 1]
  ]

Sage example in ./linalg.tex, line 220::

  sage: B[1,2]
  [0 0 0]
  [0 0 1]

Sage example in ./linalg.tex, line 260::

  sage: A = matrix(GF(11), 2, 2, [1,0,0,2])
  sage: B = matrix(GF(11), 2, 2, [0,1,1,0])
  sage: MG = MatrixGroup([A,B])
  sage: MG.cardinality()
  200
  sage: identity_matrix(GF(11),2) in MG
  True

Sage example in ./linalg.tex, line 292::

  sage: MS = MatrixSpace(ZZ,2,3); A = MS([1,2,3,4,5,6]); A
  [1 2 3]
  [4 5 6]

Sage example in ./linalg.tex, line 323::

  sage: a = matrix(); a.parent()
  Full MatrixSpace of 0 by 0 dense matrices over Integer Ring

Sage example in ./linalg.tex, line 341::

  sage: a = matrix(GF(8,'x'),3,4); a.parent()
  Full MatrixSpace of 3 by 4 dense matrices over Finite Field
  in x of size 2^3

Sage example in ./linalg.tex, line 356::

  sage: g = graphs.PetersenGraph()
  sage: m = matrix(g); m; m.parent()
  [0 1 0 0 1 1 0 0 0 0]
  [1 0 1 0 0 0 1 0 0 0]
  [0 1 0 1 0 0 0 1 0 0]
  [0 0 1 0 1 0 0 0 1 0]
  [1 0 0 1 0 0 0 0 0 1]
  [1 0 0 0 0 0 0 1 1 0]
  [0 1 0 0 0 0 0 0 1 1]
  [0 0 1 0 0 1 0 0 0 1]
  [0 0 0 1 0 1 1 0 0 0]
  [0 0 0 0 1 0 1 1 0 0]
  Full MatrixSpace of 10 by 10 dense matrices over Integer Ring

Sage example in ./linalg.tex, line 380::

  sage: A = matrix([[1,2],[3,4]])
  sage: block_matrix([[A,-A],[2*A, A^2]])
  [ 1  2|-1 -2]
  [ 3  4|-3 -4]
  [-----+-----]
  [ 2  4| 7 10]
  [ 6  8|15 22]

Sage example in ./linalg.tex, line 413::

  sage: A = matrix([[1,2,3],[4,5,6]])
  sage: block_matrix([1,A,0,0,-A,2], ncols=3)
  [ 1  0| 1  2  3| 0  0]
  [ 0  1| 4  5  6| 0  0]
  [-----+--------+-----]
  [ 0  0|-1 -2 -3| 2  0]
  [ 0  0|-4 -5 -6| 0  2]

Sage example in ./linalg.tex, line 443::

  sage: A = matrix([[1,2,3],[0,1,0]])
  sage: block_diagonal_matrix(A, A.transpose())
  [1 2 3|0 0]
  [0 1 0|0 0]
  [-----+---]
  [0 0 0|1 0]
  [0 0 0|2 1]
  [0 0 0|3 0]

Sage example in ./linalg.tex, line 517::

  sage: A = matrix(3,3,range(9))
  sage: A[:,1] = vector([1,1,1]); A
  [0 1 2]
  [3 1 5]
  [6 1 8]

Sage example in ./linalg.tex, line 535::

  sage: A[::-1], A[:,::-1], A[::2,-1]
  (
  [6 1 8]  [2 1 0]
  [3 1 5]  [5 1 3]  [2]
  [0 1 2], [8 1 6], [8]
  )

Sage example in ./linalg.tex, line 571::

  sage: A = matrix(ZZ,4,4,range(16)); A
  [ 0  1  2  3]
  [ 4  5  6  7]
  [ 8  9 10 11]
  [12 13 14 15]


Sage example in ./linalg.tex, line 587::

  sage: A.matrix_from_rows_and_columns([0,2,3],[1,2])
  [ 1  2]
  [ 9 10]
  [13 14]

Sage example in ./linalg.tex, line 639::

  sage: MS = MatrixSpace(GF(3),2,3)
  sage: MS.base_extend(GF(9,'x'))
  Full MatrixSpace of 2 by 3 dense matrices over Finite Field
  in x of size 3^2
  sage: MS = MatrixSpace(ZZ,2,3)
  sage: MS.change_ring(GF(3))
  Full MatrixSpace of 2 by 3 dense matrices over Finite Field of size 3

Sage example in ./linalg.tex, line 1167::

  sage: a = matrix(GF(7),4,3,[6,2,2,5,4,4,6,4,5,5,1,3]); a
  [6 2 2]
  [5 4 4]
  [6 4 5]
  [5 1 3]

Sage example in ./linalg.tex, line 1181::

  sage: u = copy(identity_matrix(GF(7),4)); u[1:,0] = -a[1:,0]/a[0,0]
  sage: u, u*a
  (
  [1 0 0 0]  [6 2 2]
  [5 1 0 0]  [0 0 0]
  [6 0 1 0]  [0 2 3]
  [5 0 0 1], [0 4 6]
  )

Sage example in ./linalg.tex, line 1203::

  sage: v = copy(identity_matrix(GF(7),4)); v.swap_rows(1,2)
  sage: b = v*u*a; v, b
  (
  [1 0 0 0]  [6 2 2]
  [0 0 1 0]  [0 2 3]
  [0 1 0 0]  [0 0 0]
  [0 0 0 1], [0 4 6]
  )

Sage example in ./linalg.tex, line 1225::

  sage: w = copy(identity_matrix(GF(7),4))
  sage: w[2:,1] = -b[2:,1]/b[1,1]; w, w*b
  (
  [1 0 0 0]  [6 2 2]
  [0 1 0 0]  [0 2 3]
  [0 0 1 0]  [0 0 0]
  [0 5 0 1], [0 0 0]
  )

Sage example in ./linalg.tex, line 1325::

  sage: A = matrix(GF(7),4,5,[4,4,0,2,4,5,1,6,5,4,1,1,0,1,0,5,1,6,6,2])
  sage: A, A.echelon_form()
  (
  [4 4 0 2 4]  [1 0 5 0 3]
  [5 1 6 5 4]  [0 1 2 0 6]
  [1 1 0 1 0]  [0 0 0 1 5]
  [5 1 6 6 2], [0 0 0 0 0]
  )

Sage example in ./linalg.tex, line 1512::

  sage: a = matrix(ZZ, 4, 6, [2,1,2,2,2,-1,1,2,-1,2,1,-1,2,1,-1,\
  ....:              -1,2,2,2,1,1,-1,-1,-1]); a.echelon_form()
  [ 1  2  0  5  4 -1]
  [ 0  3  0  2 -6 -7]
  [ 0  0  1  3  3  0]
  [ 0  0  0  6  9  3]

Sage example in ./linalg.tex, line 1528::

  sage: a.base_extend(QQ).echelon_form()
  [   1    0    0    0  5/2 11/6]
  [   0    1    0    0   -3 -8/3]
  [   0    0    1    0 -3/2 -3/2]
  [   0    0    0    1  3/2  1/2]

Sage example in ./linalg.tex, line 1561::

  sage: A = matrix(ZZ,4,5,[4,4,0,2,4,5,1,6,5,4,1,1,0,1,0,5,1,6,6,2])
  sage: H, U = A.echelon_form(transformation=True); H, U
  (
  [ 1  1  0  0  2]  [ 0  1  1 -1]
  [ 0  4 -6  0 -4]  [ 0 -1  5  0]
  [ 0  0  0  1 -2]  [ 0 -1  0  1]
  [ 0  0  0  0  0], [ 1 -2 -4  2]
  )

Sage example in ./linalg.tex, line 1640::

  sage: A = matrix(ZZ, 4, 5,\
  ....:            [-1,-1,-1,-2,-2,-2,1,1,-1,2,2,2,2,2,-1,2,2,2,2,2])
  sage: S,U,V = A.smith_form(); S,U,V
  (
                              [ 3  1  2 -1  0]
  [1 0 0 0 0]  [ 0  0  1  0]  [ 0  0  0  0  1]
  [0 1 0 0 0]  [ 0  1  0  0]  [ 1  1  1  1 -1]
  [0 0 3 0 0]  [ 1 -2 -4  1]  [-3 -2 -3 -1  0]
  [0 0 0 6 0], [ 0  0  4 -1], [ 1  0  0 -2  0]
  )

Sage example in ./linalg.tex, line 1674::

  sage: A.elementary_divisors()
  [1, 1, 3, 6]
  sage: S == U*A*V
  True

Sage example in ./linalg.tex, line 1746::

  sage: B = matrix(GF(7),5,4,[4,5,1,5,4,1,1,1,0,6,0,6,2,5,1,6,4,4,0,2])
  sage: B.transpose().echelon_form()
  [1 0 5 0 3]
  [0 1 2 0 6]
  [0 0 0 1 5]
  [0 0 0 0 0]

Sage example in ./linalg.tex, line 1761::

  sage: B.pivot_rows()
  (0, 1, 3)
  sage: B.transpose().pivots() == B.pivot_rows()
  True

Sage example in ./linalg.tex, line 1818::

  sage: R.<x> = PolynomialRing(GF(5),'x')
  sage: A = random_matrix(R,2,3); A                    # random
  [      3*x^2 + x       x^2 + 2*x       2*x^2 + 2]
  [    x^2 + x + 2 2*x^2 + 4*x + 3   x^2 + 4*x + 3]
  sage: while A.rank() < 2:
  ....:     A = random_matrix(R,2,3)

Sage example in ./linalg.tex, line 1830::

  sage: b = random_matrix(R,2,1); b                    # random
  [  4*x^2 + 1]
  [3*x^2 + 2*x]

Sage example in ./linalg.tex, line 1841::

  sage: A.solve_right(b)                               # random
  [(4*x^3 + 2*x + 4)/(3*x^3 + 2*x^2 + 2*x)]
  [  (3*x^2 + 4*x + 3)/(x^3 + 4*x^2 + 4*x)]
  [                                      0]

Sage example in ./linalg.tex, line 1855::

  sage: A.solve_right(b) == A\b
  doctest:...: DeprecationWarning: the backslash operator has been deprecated; use A.solve_right(B) instead
  See https://github.com/sagemath/sage/issues/36394 for details.
  True

Sage example in ./linalg.tex, line 1910::

  sage: a = matrix(QQ,3,5,[2,2,-1,-2,-1,2,-1,1,2,-1/2,2,-2,-1,2,-1/2])
  sage: a.image()
  Vector space of degree 5 and dimension 3 over Rational Field
  Basis matrix:
  [     1      0      0    1/4 -11/32]
  [     0      1      0     -1   -1/8]
  [     0      0      1    1/2   1/16]
  sage: a.right_kernel()
  Vector space of degree 5 and dimension 2 over Rational Field
  Basis matrix:
  [    1     0     0  -1/3   8/3]
  [    0     1  -1/2 11/12   2/3]

Sage example in ./linalg.tex, line 1941::

  sage: a = matrix(ZZ,5,3,[1,1,122,-1,-2,1,-188,2,1,1,-10,1,-1,-1,-1])
  sage: a.kernel()
  Free module of degree 5 and rank 2 over Integer Ring
  Echelon basis matrix:
  [   1  979  -11 -279  811]
  [   0 2079  -22 -569 1488]
  sage: b = a.base_extend(QQ)
  sage: b.kernel()
  Vector space of degree 5 and dimension 2 over Rational Field
  Basis matrix:
  [        1         0  -121/189 -2090/189   6949/63]
  [        0         1    -2/189 -569/2079   496/693]
  sage: b.integer_kernel()
  Free module of degree 5 and rank 2 over Integer Ring
  Echelon basis matrix:
  [   1  979  -11 -279  811]
  [   0 2079  -22 -569 1488]

Sage example in ./linalg.tex, line 2301::

  sage: A = matrix(GF(97), 4, 4,\
  ....:            [86,1,6,68,34,24,8,35,15,36,68,42,27,1,78,26])
  sage: e1 = identity_matrix(GF(97),4)[0]
  sage: U = matrix(A.transpose().maxspin(e1)).transpose()
  sage: F = U^-1*A*U; F
  [ 0  0  0 83]
  [ 1  0  0 77]
  [ 0  1  0 20]
  [ 0  0  1 10]

Sage example in ./linalg.tex, line 2320::

  sage: K.<x> = GF(97)[]
  sage: P = x^4-sum(F[i,3]*x^i for i in range(4)); P
  x^4 + 87*x^3 + 77*x^2 + 20*x + 14

Sage example in ./linalg.tex, line 2326::

  sage: P == A.charpoly()
  True

Sage example in ./linalg.tex, line 2502::

  sage: A = matrix(ZZ,8,[[6,0,-2,4,0,0,0,-2],[14,-1,0,6,0,-1,-1,1],\
  ....:                  [2,2,0,1,0,0,1,0],[-12,0,5,-8,0,0,0,4],\
  ....:                  [0,4,0,0,0,0,4,0],[0,0,0,0,1,0,0,0],\
  ....:                  [-14,2,0,-6,0,2,2,-1],[-4,0,2,-4,0,0,0,4]])
  sage: A.frobenius_form()
  [0 0 0 4 0 0 0 0]
  [1 0 0 4 0 0 0 0]
  [0 1 0 1 0 0 0 0]
  [0 0 1 0 0 0 0 0]
  [0 0 0 0 0 0 4 0]
  [0 0 0 0 1 0 0 0]
  [0 0 0 0 0 1 1 0]
  [0 0 0 0 0 0 0 2]

Sage example in ./linalg.tex, line 2540::

  sage: A.frobenius_form(1)
  [x^4 - x^2 - 4*x - 4, x^3 - x^2 - 4, x - 2]

Sage example in ./linalg.tex, line 2547::

  sage: F,K = A.frobenius_form(2)
  sage: K
  [      1  -15/56  17/224   15/56 -17/896       0 -15/112   17/64]
  [      0  29/224 -13/224 -23/448 -17/896 -17/896  29/448  13/128]
  [      0 -75/896  75/896 -47/896       0 -17/896 -23/448  11/128]
  [      0  17/896 -29/896  15/896       0       0       0       0]
  [      0       0       0       0       1       0       0       0]
  [      0       0       0       0       0       1       0       0]
  [      0       1       0       0       0       0       1       0]
  [      0   -4/21   -4/21  -10/21       0       0   -2/21       1]

Sage example in ./linalg.tex, line 2573::

  sage: K^-1*F*K == A
  True

Sage example in ./linalg.tex, line 2621::

  sage: S.<x> = QQ[]
  sage: B = x*identity_matrix(8) - A
  sage: B.elementary_divisors()
  [1, 1, 1, 1, 1, x - 2, x^3 - x^2 - 4, x^4 - x^2 - 4*x - 4]

Sage example in ./linalg.tex, line 2629::

  sage: A.frobenius_form(1)
  [x^4 - x^2 - 4*x - 4, x^3 - x^2 - 4, x - 2]

Sage example in ./linalg.tex, line 2726::

  sage: A = matrix(GF(7),4,[5,5,4,3,0,3,3,4,0,1,5,4,6,0,6,3])
  sage: A.eigenvalues()
  [4, 1, 2, 2]
  sage: A.eigenvectors_right()
  [(4, [
  (1, 5, 5, 1)
  ], 1), (1, [
  (0, 1, 1, 4)
  ], 1), (2, [
  (1, 3, 0, 1),
  (0, 0, 1, 1)
  ], 2)]
  sage: A.eigenspaces_right()
  [
  (4, Vector space of degree 4 and dimension 1 over Finite Field
  of size 7
  User basis matrix:
  [1 5 5 1]),
  (1, Vector space of degree 4 and dimension 1 over Finite Field
  of size 7
  User basis matrix:
  [0 1 1 4]),
  (2, Vector space of degree 4 and dimension 2 over Finite Field
  of size 7
  User basis matrix:
  [1 3 0 1]
  [0 0 1 1])
  ]

Sage example in ./linalg.tex, line 2770::

  sage: A.eigenmatrix_right()
  (
  [4 0 0 0]  [1 0 1 0]
  [0 1 0 0]  [5 1 3 0]
  [0 0 2 0]  [5 1 0 1]
  [0 0 0 2], [1 4 1 1]
  )

Sage example in ./linalg.tex, line 2930::

  sage: A = matrix(ZZ,4,[3,-1,0,-1,0,2,0,-1,1,-1,2,0,1,-1,-1,3])
  sage: A.jordan_form()
  [3|0|0 0]
  [-+-+---]
  [0|3|0 0]
  [-+-+---]
  [0|0|2 1]
  [0|0|0 2]

Sage example in ./linalg.tex, line 2949::

  sage: J,U = A.jordan_form(transformation=True)
  sage: U^-1*A*U == J
  True
"""



"""
This file (./nonlinear_doctest.sage) was *autogenerated* from ./nonlinear.tex,
with sagetex.sty version 2011/05/27 v2.3.1.
It contains the contents of all the sageexample environments from this file.
You should be able to doctest this file with:
sage -t ./nonlinear_doctest.sage
It is always safe to delete this file; it is not used in typesetting your
document.

Sage example in ./nonlinear.tex, line 102::

  sage: R.<x> = PolynomialRing(RealField(prec=10))
  sage: p = 2*x^7 - 21*x^6 + 64*x^5 - 67*x^4 + 90*x^3 \
  ....: + 265*x^2 - 900*x + 375
  sage: p.roots()
  [(-1.7, 1), (0.50, 1), (1.7, 1), (5.0, 2)]
  sage: p.roots(ring=ComplexField(10), multiplicities=False)
  [-1.7, 0.50, 1.7, 5.0, -2.2*I, 2.2*I]
  sage: p.roots(ring=RationalField())
  [(1/2, 1), (5, 2)]

Sage example in ./nonlinear.tex, line 246::

  sage: R.<x> = PolynomialRing(QQ, 'x')
  sage: p = x^4 + x^3 + x^2 + x + 1
  sage: K.<alpha> = p.root_field()
  sage: p.roots(ring=K, multiplicities=None)
  [alpha, alpha^2, alpha^3, -alpha^3 - alpha^2 - alpha - 1]
  sage: alpha^5
  1

Sage example in ./nonlinear.tex, line 309::

  sage: R.<x> = PolynomialRing(RR, 'x')
  sage: d = ZZ.random_element(1, 15)
  sage: p = R.random_element(d)
  sage: p.degree() == sum(r[1] for r in p.roots(CC))
  True

Sage example in ./nonlinear.tex, line 348::

  sage: def build_complex_roots(degree):
  ....:     R.<x> = PolynomialRing(CDF, 'x')
  ....:     v = []
  ....:     for c in cartesian_product([[-1, 1]] * (degree + 1)):
  ....:         v.extend(R(list(c)).roots(multiplicities=False))
  ....:     return v
  sage: data = build_complex_roots(12)  # long time
  sage: points(data, pointsize=1, aspect_ratio=1)  # long time
  Graphics object consisting of 1 graphics primitive

Sage example in ./nonlinear.tex, line 420::

  sage: a, b, c, x = var('a, b, c, x')
  sage: p = a * x^2 + b * x + c
  sage: type(p)
  <... 'sage.symbolic.expression.Expression'>
  sage: p.parent()
  Symbolic Ring
  sage: p.roots(x)
  [(-1/2*(b + sqrt(b^2 - 4*a*c))/a, 1),
   (-1/2*(b - sqrt(b^2 - 4*a*c))/a, 1)]

Sage example in ./nonlinear.tex, line 450::

  sage: a, b, c, d, e, f, x = var('a, b, c, d, e, f, x')
  sage: p = a*x^5+b*x^4+c*x^3+d*x^2+e*x+f
  sage: p.roots(x)
  Traceback (most recent call last):
  ...
  RuntimeError: no explicit roots found

Sage example in ./nonlinear.tex, line 467::

  sage: x, a, b, c, d = var('x, a, b, c, d')
  sage: P = a * x^3 + b * x^2 + c * x + d
  sage: alpha = var('alpha')
  sage: P.subs(x = x + alpha).expand().coefficient(x, 2)
  3*a*alpha + b
  sage: P.subs(x = x - b / (3 * a)).expand().collect(x)
  a*x^3 - 1/3*(b^2/a - 3*c)*x + 2/27*b^3/a^2 - 1/3*b*c/a + d

Sage example in ./nonlinear.tex, line 482::

  sage: p, q, u, v = var('p, q, u, v')
  sage: P = x^3 + p * x + q
  sage: P.subs(x = u + v).expand()
  u^3 + 3*u^2*v + 3*u*v^2 + v^3 + p*u + p*v + q

Sage example in ./nonlinear.tex, line 498::

  sage: P.subs({x: u + v, q: -u^3 - v^3}).factor()
  (3*u*v + p)*(u + v)
  sage: P.subs({x: u+v, q: -u^3 - v^3, p: -3 * u * v}).expand()
  0
  sage: X = var('X')
  sage: solve([X^2 + q*X - p^3 / 27 == 0], X, solution_dict=True)
  [{X: -1/2*q - 1/18*sqrt(12*p^3 + 81*q^2)},
   {X: -1/2*q + 1/18*sqrt(12*p^3 + 81*q^2)}]

Sage example in ./nonlinear.tex, line 539::

  sage: e = sin(x) * (x^3 + 1) * (x^5 + x^4 + 1)
  sage: roots = e.roots(); len(roots)
  9
  sage: roots
  [(0, 1),
   (-1/2*(1/18*sqrt(23)*sqrt(3) - 1/2)^(1/3)*(I*sqrt(3) + 1) - 1/6*(-I*sqrt(3) + 1)/(1/18*sqrt(23)*sqrt(3) - 1/2)^(1/3),
    1),
   (-1/2*(1/18*sqrt(23)*sqrt(3) - 1/2)^(1/3)*(-I*sqrt(3) + 1) - 1/6*(I*sqrt(3) + 1)/(1/18*sqrt(23)*sqrt(3) - 1/2)^(1/3),
    1),
   ((1/18*sqrt(23)*sqrt(3) - 1/2)^(1/3) + 1/3/(1/18*sqrt(23)*sqrt(3) - 1/2)^(1/3),
    1),
   (-1/2*I*sqrt(3) - 1/2, 1),
   (1/2*I*sqrt(3) - 1/2, 1),
   (1/2*I*sqrt(3)*(-1)^(1/3) - 1/2*(-1)^(1/3), 1),
   (-1/2*I*sqrt(3)*(-1)^(1/3) - 1/2*(-1)^(1/3), 1),
   ((-1)^(1/3), 1)]

Sage example in ./nonlinear.tex, line 620::

  sage: alpha, m, x = var('alpha, m, x'); q = function('q')(x)
  sage: p = (x - alpha)^m * q
  sage: p.derivative(x)
  (-alpha + x)^(m - 1)*m*q(x) + (-alpha + x)^m*diff(q(x), x)
  sage: simplify(p.derivative(x)(x=alpha))
  0

Sage example in ./nonlinear.tex, line 659::

  sage: R.<x> = PolynomialRing(QQ, 'x')
  sage: p = 128 * x^13 - 1344 * x^12 + 6048 * x^11 - 15632 * x^10 \
  ....: + 28056 * x^9 - 44604 * x^8 + 71198 * x^7 - 98283 * x^6   \
  ....: + 105840 * x^5 - 101304 * x^4 + 99468 * x^3 - 81648 * x^2 \
  ....: + 40824 * x - 8748
  sage: d = gcd(p, p.derivative())
  sage: (p // d).degree()
  4
  sage: roots = SR(p // d).roots(multiplicities=False)
  sage: roots
  [1/2*I*sqrt(3)*2^(1/3) - 1/2*2^(1/3),
   -1/2*I*sqrt(3)*2^(1/3) - 1/2*2^(1/3),
   2^(1/3),
   3/2]
  sage: [QQbar(p(alpha)).is_zero() for alpha in roots]  # long time
  [True, True, True, True]

Sage example in ./nonlinear.tex, line 732::

  sage: R.<x> = PolynomialRing(RR, 'x')
  sage: p = x^7 - 131/3*x^6 + 1070/3*x^5 - 2927/3*x^4 \
  ....: + 2435/3*x^3 - 806/3*x^2 + 3188/3*x - 680
  sage: l = [c for c in p.coefficients(sparse=False) if not c.is_zero()]
  sage: sign_changes = [l[i]*l[i+1] < 0 for i in range(len(l)-1)].count(True)
  sage: real_positive_roots = \
  ....: sum([alpha[1] if alpha[0] > 0 else 0 for alpha in p.roots()])
  sage: sign_changes, real_positive_roots
  (7, 5)

Sage example in ./nonlinear.tex, line 831::

  sage: def count_sign_changes(p):
  ....:     l = [c for c in p if not c.is_zero()]
  ....:     changes = [l[i]*l[i + 1] < 0 for i in range(len(l) - 1)]
  ....:     return changes.count(True)

Sage example in ./nonlinear.tex, line 837::

  sage: def sturm(p, a, b):
  ....:     assert p.degree() > 2
  ....:     assert not (p(a) == 0)
  ....:     assert not (p(b) == 0)
  ....:     assert a <= b
  ....:     remains = [p, p.derivative()]
  ....:     for i in range(p.degree() - 1):
  ....:         remains.append(-(remains[i] % remains[i + 1]))
  ....:     evals = [[], []]
  ....:     for q in remains:
  ....:         evals[0].append(q(a))
  ....:         evals[1].append(q(b))
  ....:     return count_sign_changes(evals[0]) \
  ....:            - count_sign_changes(evals[1])

Sage example in ./nonlinear.tex, line 857::

  sage: R.<x> = PolynomialRing(QQ, 'x')
  sage: p = (x - 34) * (x - 5) * (x - 3) * (x - 2) * (x - 2/3)
  sage: sturm(p, 1, 4)
  2
  sage: sturm(p, 1, 10)
  3
  sage: sturm(p, 1, 200)
  4
  sage: p.roots(multiplicities=False)
  [34, 5, 3, 2, 2/3]
  sage: sturm(p, 1/2, 35)
  5

Sage example in ./nonlinear.tex, line 949::

  sage: f(x) = 4 * sin(x) - exp(x) / 2 + 1
  sage: a, b = RR(-pi), RR(pi)
  sage: bool(f(a) * f(b) < 0)
  True

Sage example in ./nonlinear.tex, line 961::

  sage: solve(f(x) == 0, x)
  [sin(x) == 1/8*e^x - 1/4]

Sage example in ./nonlinear.tex, line 966::

  sage: f.roots()
  Traceback (most recent call last):
   ...
  RuntimeError: no explicit roots found

Sage example in ./nonlinear.tex, line 991::

  sage: a, b = RR(-pi), RR(pi)
  sage: g = plot(f, a, b, rgbcolor='blue')

Sage example in ./nonlinear.tex, line 1045::

  sage: def phi(s, t): return (s + t) / 2
  sage: def intervalgen(f, phi, s, t):
  ....:     msg = 'Wrong arguments: f({0})*f({1})>=0)'.format(s, t)
  ....:     assert (f(s) * f(t) < 0), msg
  ....:     yield s
  ....:     yield t
  ....:     while True:
  ....:         u = phi(s, t)
  ....:         yield u
  ....:         if f(u) * f(s) < 0:
  ....:             t = u
  ....:         else:
  ....:             s = u

Sage example in ./nonlinear.tex, line 1152::

  sage: a, b
  (-3.14159265358979, 3.14159265358979)
  sage: bisection = intervalgen(f, phi, a, b)
  sage: next(bisection)
  -3.14159265358979
  sage: next(bisection)
  3.14159265358979
  sage: next(bisection)
  0.000000000000000

Sage example in ./nonlinear.tex, line 1178::

  sage: from types import GeneratorType, FunctionType
  sage: def checklength(u, v, w, prec):
  ....:     return abs(v - u) < 2 * prec
  sage: def iterate(series, check=checklength, prec=10^-5, maxit=100):
  ....:     assert isinstance(series, GeneratorType)
  ....:     assert isinstance(check, FunctionType)
  ....:     niter = 2
  ....:     v, w = next(series), next(series)
  ....:     while niter <= maxit:
  ....:         niter += 1
  ....:         u, v, w = v, w, next(series)
  ....:         if check(u, v, w, prec):
  ....:             print('After {0} iterations: {1}'.format(niter, w))
  ....:             return
  ....:     print('Failed after {0} iterations'.format(maxit))

Sage example in ./nonlinear.tex, line 1221::

  sage: bisection = intervalgen(f, phi, a, b)
  sage: iterate(bisection)
  After 22 iterations: 2.15847275559132

Sage example in ./nonlinear.tex, line 1313::

  sage: phi(s, t) = s - f(s) * (t - s) / (f(t) - f(s))
  sage: falsepos = intervalgen(f, phi, a, b)
  sage: iterate(falsepos)
  After 8 iterations: -2.89603757331027

Sage example in ./nonlinear.tex, line 1320::

  sage: a, b = RR(-pi), RR(pi)
  sage: g = plot(f, a, b, rgbcolor='blue')
  sage: phi(s, t) = s - f(s) * (t - s) /(f(t) - f(s))
  sage: falsepos = intervalgen(f, phi, a, b)
  sage: u, v, w = next(falsepos), next(falsepos), next(falsepos)
  sage: niter = 3
  sage: while niter < 9:
  ....:     g += line([(u, 0), (u, f(u))], rgbcolor='red',
  ....:               linestyle=':')
  ....:     g += line([(u, f(u)), (v, f(v))], rgbcolor='red')
  ....:     g += line([(v, 0), (v, f(v))], rgbcolor='red',
  ....:               linestyle=':')
  ....:     g += point((w, 0), rgbcolor='red')
  ....:     if (f(u) * f(w)) < 0:
  ....:         u, v = u, w
  ....:     else:
  ....:         u, v = w, v
  ....:     w = next(falsepos)
  ....:     niter += 1

Sage example in ./nonlinear.tex, line 1361::

  sage: a, b = RR(pi/2), RR(pi)
  sage: phi(s, t) = t - f(t) * (s - t) / (f(s) - f(t))
  sage: falsepos = intervalgen(f, phi, a, b)
  sage: phi(s, t) = (s + t) / 2
  sage: bisection = intervalgen(f, phi, a, b)
  sage: iterate(falsepos)
  After 15 iterations: 2.15846441170219
  sage: iterate(bisection)
  After 20 iterations: 2.15847275559132

Sage example in ./nonlinear.tex, line 1373::

  sage: a, b = RR(pi/2), RR(pi)
  sage: g = plot(f, a, b, rgbcolor='blue')
  sage: phi(s, t) = t - f(t) * (s - t) / (f(s) - f(t))
  sage: falsepos = intervalgen(f, phi, a, b)
  sage: u, v, w = next(falsepos), next(falsepos), next(falsepos)
  sage: niter = 3
  sage: while niter < 7:
  ....:     g += line([(u, 0), (u, f(u))], rgbcolor='red',
  ....:               linestyle=':')
  ....:     g += line([(u, f(u)), (v, f(v))], rgbcolor='red')
  ....:     g += line([(v, 0), (v, f(v))], rgbcolor='red',
  ....:               linestyle=':')
  ....:     g += point((w, 0), rgbcolor='red')
  ....:     if (f(u) * f(w)) < 0:
  ....:         u, v = u, w
  ....:     else:
  ....:         u, v = w, v
  ....:     w = next(falsepos)
  ....:     niter += 1

Sage example in ./nonlinear.tex, line 1477::

  sage: f.derivative()
  x |--> 4*cos(x) - 1/2*e^x
  sage: a, b = RR(pi/2), RR(pi)

Sage example in ./nonlinear.tex, line 1501::

  sage: def newtongen(f, u):
  ....:     while True:
  ....:         yield u
  ....:         u -= f(u) / f.derivative()(u)
  sage: def checkconv(u, v, w, prec):
  ....:     return abs(w - v) / abs(w) <= prec

Sage example in ./nonlinear.tex, line 1513::

  sage: iterate(newtongen(f, a), check=checkconv)
  After 6 iterations: 2.15846852566756

Sage example in ./nonlinear.tex, line 1518::

  sage: generator = newtongen(f, a)
  sage: g = plot(f, a, b, rgbcolor='blue')
  sage: u, v = next(generator), next(generator)
  sage: niter = 2
  sage: while niter < 6:
  ....:     g += point((u, 0), rgbcolor='red')
  ....:     g += line([(u, 0), (u, f(u))], rgbcolor='red',
  ....:               linestyle=':')
  ....:     g += line([(u, f(u)), (v, 0)], rgbcolor='red')
  ....:     u, v = v, next(generator)
  ....:     niter += 1

Sage example in ./nonlinear.tex, line 1583::

  sage: def secantgen(f, a):
  ....:     yield a
  ....:     estimate = f.derivative()(a)
  ....:     b = a - f(a) / estimate
  ....:     yield b
  ....:     while True:
  ....:         fa, fb = f(a), f(b)
  ....:         if fa == fb:
  ....:             estimate = f.derivative()(a)
  ....:         else:
  ....:             estimate = (fb - fa) / (b - a)
  ....:         a = b
  ....:         b -= fb / estimate
  ....:         yield b

Sage example in ./nonlinear.tex, line 1609::

  sage: iterate(secantgen(f, a), check=checkconv)
  After 8 iterations: 2.15846852557553

Sage example in ./nonlinear.tex, line 1621::

  sage: g = plot(f, a, b, rgbcolor='blue')
  sage: sequence = secantgen(f, a)
  sage: u, v = next(sequence), next(sequence)
  sage: niter = 2
  sage: while niter < 6:
  ....:     g += point((u, 0), rgbcolor='red')
  ....:     g += line([(u, 0), (u, f(u))], rgbcolor='red',
  ....:               linestyle=':')
  ....:     g += line([(u, f(u)), (v, 0)], rgbcolor='red')
  ....:     u, v = v, next(sequence)
  ....:     niter += 1

Sage example in ./nonlinear.tex, line 1697::

  sage: from collections import deque
  sage: basering = PolynomialRing(CC, 'x')
  sage: def quadraticgen(f, r, s):
  ....:     t = (r + s) / 2
  ....:     yield t
  ....:     points = deque([(r,f(r)), (s,f(s)), (t,f(t))], maxlen=int(3))
  ....:     while True:
  ....:         pol = basering.lagrange_polynomial(points)
  ....:         roots = pol.roots(ring=CC, multiplicities=False)
  ....:         u = min(roots, key=lambda x: abs(x - points[2][0]))
  ....:         points.append((u, f(u)))
  ....:         yield points[2][0]

Sage example in ./nonlinear.tex, line 1742::

  sage: generator = quadraticgen(f, a, b)
  sage: iterate(generator, check=checkconv)
  After 5 iterations: 2.15846852554764

Sage example in ./nonlinear.tex, line 1851::

  sage: for ring in [ZZ, QQ, QQbar, RDF, RIF, RR, AA, CDF, CIF, CC]:
  ....:     print("{0:50} {1}".format(str(ring), ring.is_exact()))
  Integer Ring                                       True
  Rational Field                                     True
  Algebraic Field                                    True
  Real Double Field                                  False
  Real Interval Field with 53 bits of precision      False
  Real Field with 53 bits of precision               False
  Algebraic Real Field                               True
  Complex Double Field                               False
  Complex Interval Field with 53 bits of precision   False
  Complex Field with 53 bits of precision            False

Sage example in ./nonlinear.tex, line 2031::

  sage: def steffensen(sequence):
  ....:     assert isinstance(sequence, GeneratorType)
  ....:     values = deque(maxlen=int(3))
  ....:     for i in range(3):
  ....:         values.append(next(sequence))
  ....:         yield values[i]
  ....:     while True:
  ....:         values.append(next(sequence))
  ....:         u, v, w = values
  ....:         yield u - (v - u)^2 / (w - 2 * v + u)

Sage example in ./nonlinear.tex, line 2047::

  sage: g(x) = sin(x^2 - 2) * (x^2 - 2)
  sage: sequence = newtongen(g, RR(0.7))
  sage: accelseq = steffensen(newtongen(g, RR(0.7)))
  sage: iterate(sequence, check=checkconv)
  After 17 iterations: 1.41422192763287
  sage: iterate(accelseq, check=checkconv)
  After 10 iterations: 1.41421041980166

Sage example in ./nonlinear.tex, line 2062::

  sage: sequence = newtongen(f, RR(a))
  sage: accelseq = steffensen(newtongen(f, RR(a)))
  sage: iterate(sequence, check=checkconv)
  After 6 iterations: 2.15846852566756
  sage: iterate(accelseq, check=checkconv)
  After 7 iterations: 2.15846852554764

Sage example in ./nonlinear.tex, line 2124::

  sage: result = (f == 0).find_root(a, b, full_output=True)
  sage: result[0], result[1].iterations
  (2.1584685255476415, 9)

Sage example in ./nonlinear.tex, line 2183::

  sage: a, b = pi/2, pi
  sage: generator = newtongen(f, a)
  sage: next(generator); next(generator)
  1/2*pi
  1/2*pi - (e^(1/2*pi) - 10)*e^(-1/2*pi)
"""



"""
Interactively tracing execution of a command
"""


def trace(code, preparse=True):
    r"""
    Evaluate Sage code using the interactive tracer and return the
    result. The string ``code`` must be a valid expression
    enclosed in quotes (no assignments - the result of the expression
    is returned). In the Sage notebook this just raises a
    NotImplementedException.

    INPUT:

    - ``code`` -- string

    - ``preparse`` -- boolean (default: ``True``); if ``True``, run
      expression through the Sage preparser

    REMARKS: This function is extremely powerful! For example, if you
    want to step through each line of execution of, e.g.,
    ``factor(100)``, type

    ::

        sage: from sage.misc.trace import trace
        sage: trace("factor(100)")             # not tested

    then at the (Pdb) prompt type ``s`` (or ``step``), then press :kbd:`Return`
    over and over to step through every line of Python that is called
    in the course of the above computation. Type ``?`` at any time for
    help on how to use the debugger (e.g., ``l`` lists 11 lines around
    the current line; ``bt`` gives a back trace, etc.).

    Setting a break point: If you have some code in a file and would
    like to drop into the debugger at a given point, put the following
    code at that point in the file:

    ``import pdb; pdb.set_trace()``

    For an article on how to use the Python debugger, see
    http://www.onlamp.com/pub/a/python/2005/09/01/debugger.html

    TESTS:

    For tests we disable garbage collection, see :issue:`21258` ::

        sage: import gc
        sage: gc.disable()

    The only real way to test this is via pexpect spawning a
    sage subprocess that uses IPython::

        sage: # needs pexpect sage.all
        sage: import pexpect
        sage: s = pexpect.spawn('sage')
        sage: _ = s.sendline("from sage.misc.trace import trace; trace('print(factor(10))'); print(3+97)")
        sage: _ = s.expect('ipdb>', timeout=90)
        sage: _ = s.sendline("s"); _ = s.sendline("c")
        sage: _ = s.expect('100', timeout=90)

    Seeing the ipdb prompt and the 2 \* 5 in the output below is a
    strong indication that the trace command worked correctly::

        sage: print(s.before[s.before.find(b'--'):].decode())                           # needs pexpect sage.all
        --...
        ...ipdb> c
        ...2 * 5...

    Re-enable garbage collection::

        sage: gc.enable()
    """
    from IPython.core.debugger import Pdb
    pdb = Pdb()

    try:
        ipython = get_ipython()
    except NameError:
        raise NotImplementedError("the trace command can only be run from the Sage command-line")

    from sage.repl.preparse import preparse
    code = preparse(code)
    return pdb.run(code, ipython.user_ns)

    # this could also be useful; it drops
    # us into a debugger in an except block:
    #     import pdb; pdb.post_mortem(sys.exc_info()[2])



##          Sage Doctest File         ##
#**************************************#
#*    Generated from PreTeXt source   *#
#*    on 2017-08-24T11:43:34-07:00    *#
#*                                    *#
#*   http://mathbook.pugetsound.edu   *#
#*                                    *#
#**************************************#
##
"""
Please contact Rob Beezer (beezer@ups.edu) with
any test failures here that need to be changed
as a result of changes accepted into Sage.  You
may edit/change this file in any sensible way, so
that development work may procede.  Your changes
may later be replaced by the authors of "Abstract
Algebra: Theory and Applications" when the text is
updated, and a replacement of this file is proposed
for review.
"""
##
## To execute doctests in these files, run
##   $ $SAGE_ROOT/sage -t <directory-of-these-files>
## or
##   $ $SAGE_ROOT/sage -t <a-single-file>
##
## Replace -t by "-tp n" for parallel testing,
##   "-tp 0" will use a sensible number of threads
##
## See: http://www.sagemath.org/doc/developer/doctesting.html
##   or run  $ $SAGE_ROOT/sage --advanced  for brief help
##
## Generated at 2017-08-24T11:43:34-07:00
## From "Abstract Algebra"
## At commit 26d3cac0b4047f4b8d6f737542be455606e2c4b4
##
## Exercises 6.6 Sage Exercises
##
r"""
~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: a = 5
    sage: b = 10
    sage: c = 6
    sage: d = 13
    sage: a.divides(b)
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: not (b in [c,d])
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: a.divides(b) and not (b in [c,d])
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: a = 20
    sage: b = 6
    sage: a.mod(b)
    2

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: prime_range(50, 100)
    [53, 59, 61, 67, 71, 73, 79, 83, 89, 97]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: all([True, True, True, True])
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: all([True, True, False, True])
    False

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: [a/b for a in srange(9) for b in srange(1,a) if gcd(a,b)==1]
    [2, 3, 3/2, 4, 4/3, 5, 5/2, 5/3, 5/4, 6, 6/5,
     7, 7/2, 7/3, 7/4, 7/5, 7/6, 8, 8/3, 8/5, 8/7]
"""



# sage.doctest: needs sage.combinat sage.modules
r"""
Tests For Combinatorial Hopf Algebras

We check that the composition of `Sym \to FSym^* \to QSym`
agrees with `Sym \to QSym`::

    sage: s = SymmetricFunctions(QQ).s()
    sage: M = QuasiSymmetricFunctions(QQ).M()
    sage: F = algebras.FSym(QQ).dual().F()
    sage: all(F(s[p]).to_quasisymmetric_function() == M(s[p])
    ....:     for n in range(5)
    ....:     for p in Partitions(n))
    True

There are various morphisms between the Hopf algebras below.
We test that the diagram of morphisms is commutative::

    sage: NSym = NonCommutativeSymmetricFunctions(QQ)
    sage: QSym = QuasiSymmetricFunctions(QQ)
    sage: FQSym = algebras.FQSym(QQ)
    sage: FSym = algebras.FSym(QQ)
    sage: FSymDual = FSym.dual()
    sage: Sym = SymmetricFunctions(QQ)

    sage: def go(composition):
    ....:     x = NSym.a_realization()[composition]
    ....:     if QSym(FQSym(x)) != QSym(Sym(x)): return False
    ....:     if Sym(FSym(x)) != Sym(x): return False
    ....:     if FQSym(FSym(x)) != FQSym(x): return False
    ....:     if FSymDual(Sym(x)) != FSymDual(FQSym(x)): return False
    ....:     return True

    sage: go([2,1,2])  # not tested (needs more morphisms)
    True
    sage: all(all(go(comp) for comp in Compositions(n)) for n in range(5))  # not tested (needs more morphisms)
    True

    sage: def go2(n):
    ....:     for sigma in Permutations(n):
    ....:          x = FQSym.F()[sigma]
    ....:          if QSym(FSymDual(x)) != QSym(x): return False
    ....:     s = Sym.s()
    ....:     for mu in Partitions(n):
    ....:          x = s[mu]
    ....:          if QSym(FSymDual(x)) != QSym(x): return False
    ....:     return True

    sage: all(go2(n) for n in range(6))  # not tested (needs more morphisms)
    True
"""



##          Sage Doctest File         ##
#**************************************#
#*    Generated from PreTeXt source   *#
#*    on 2017-08-24T11:43:34-07:00    *#
#*                                    *#
#*   http://mathbook.pugetsound.edu   *#
#*                                    *#
#**************************************#
##
"""
Please contact Rob Beezer (beezer@ups.edu) with
any test failures here that need to be changed
as a result of changes accepted into Sage.  You
may edit/change this file in any sensible way, so
that development work may procede.  Your changes
may later be replaced by the authors of "Abstract
Algebra: Theory and Applications" when the text is
updated, and a replacement of this file is proposed
for review.
"""
##
## To execute doctests in these files, run
##   $ $SAGE_ROOT/sage -t <directory-of-these-files>
## or
##   $ $SAGE_ROOT/sage -t <a-single-file>
##
## Replace -t by "-tp n" for parallel testing,
##   "-tp 0" will use a sensible number of threads
##
## See: http://www.sagemath.org/doc/developer/doctesting.html
##   or run  $ $SAGE_ROOT/sage --advanced  for brief help
##
## Generated at 2017-08-24T11:43:34-07:00
## From "Abstract Algebra"
## At commit 26d3cac0b4047f4b8d6f737542be455606e2c4b4
##
## Section 6.5 Sage
##
r"""
~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: G = SymmetricGroup(3)
    sage: a = G("(1,2)")
    sage: H = G.subgroup([a])
    sage: rc = G.cosets(H, side='right'); rc
    [[(), (1,2)], [(2,3), (1,3,2)], [(1,2,3), (1,3)]]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: lc = G.cosets(H, side='left'); lc
    [[(), (1,2)], [(2,3), (1,2,3)], [(1,3,2), (1,3)]]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: G = SymmetricGroup(3)
    sage: b = G("(1,2,3)")
    sage: H = G.subgroup([b])
    sage: rc = G.cosets(H, side='right'); rc
    [[(), (1,2,3), (1,3,2)], [(2,3), (1,3), (1,2)]]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: lc = G.cosets(H, side='left'); lc
    [[(), (1,2,3), (1,3,2)], [(2,3), (1,2), (1,3)]]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: rc == lc
    False

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: rc_sorted = sorted([sorted(coset) for coset in rc])
    sage: rc_sorted
    [[(), (1,2,3), (1,3,2)], [(2,3), (1,2), (1,3)]]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: lc_sorted = sorted([sorted(coset) for coset in lc])
    sage: lc_sorted
    [[(), (1,2,3), (1,3,2)], [(2,3), (1,2), (1,3)]]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: rc_sorted == lc_sorted
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: G = SymmetricGroup(3)
    sage: sg = G.subgroups(); sg
    [Subgroup generated by [()] of (Symmetric group of order 3! as a permutation group),
     Subgroup generated by [(2,3)] of (Symmetric group of order 3! as a permutation group),
     Subgroup generated by [(1,2)] of (Symmetric group of order 3! as a permutation group),
     Subgroup generated by [(1,3)] of (Symmetric group of order 3! as a permutation group),
     Subgroup generated by [(1,2,3)] of (Symmetric group of order 3! as a permutation group),
     Subgroup generated by [(1,2,3), (2,3)] of (Symmetric group of order 3! as a permutation group)]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: H = sg[4]; H
    Subgroup generated by [(1,2,3)] of (Symmetric group of order 3! as a permutation group)

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: H.order()
    3

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: H.list()
    [(), (1,2,3), (1,3,2)]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: H.is_cyclic()
    True

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: G = AlternatingGroup(4)
    sage: sg = G.subgroups()
    sage: [H.order() for H in sg]
    [1, 2, 2, 2, 3, 3, 3, 3, 4, 12]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: G = SymmetricGroup(4)
    sage: sg = G.subgroups()
    sage: [H.order() for H in sg]
    [1, 2, 2, 2, 2, 2, 2, 2, 2, 2, 3, 3, 3, 3, 4, 4, 4, 4, 4, 4, 4,
     6, 6, 6, 6, 8, 8, 8, 12, 24]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: len(sg)
    30

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: G = CyclicPermutationGroup(20)
    sage: [H.order() for H in G.subgroups()]
    [1, 2, 4, 5, 10, 20]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: G = CyclicPermutationGroup(19)
    sage: [H.order() for H in G.subgroups()]
    [1, 19]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: n = 8
    sage: G = CyclicPermutationGroup(n)
    sage: [H.order() for H in G.subgroups()]
    [1, 2, 4, 8]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: euler_phi(345)
    176

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: m = random_prime(10000)
    sage: n = random_prime(10000)
    sage: m, n, euler_phi(m*n) == euler_phi(m)*euler_phi(n)   # random
    (5881, 1277, True)
"""



"""
This file (./sol/mpoly_doctest.sage) was *autogenerated* from ./sol/mpoly.tex,
with sagetex.sty version 2011/05/27 v2.3.1.
It contains the contents of all the sageexample environments from this file.
You should be able to doctest this file with:
sage -t ./sol/mpoly_doctest.sage
It is always safe to delete this file; it is not used in typesetting your
document.

Sage example in ./sol/mpoly.tex, line 27::

  sage: ring = QQ['x,y,z']; deg = 2
  sage: tmp1 = [(x,)*deg for x in (1,) + ring.gens()]; tmp1
  [(1, 1), (x, x), (y, y), (z, z)]
  sage: tmp2 = flatten(tmp1); tmp2
  [1, 1, x, x, y, y, z, z]
  sage: monomials = Subsets(tmp2, deg, submultiset=True); monomials
  SubMultiset of [1, 1, x, x, y, y, z, z] of size 2
  sage: monomials.list()
  [[1, 1], [1, x], [1, y], [1, z], [x, x], [x, y], [x, z], [y, y], [y, z], [z, z]]

Sage example in ./sol/mpoly.tex, line 98::

  sage: ['x%d' % n for n in [2,3,5,7]]
  ['x2', 'x3', 'x5', 'x7']
  sage: R = PolynomialRing(QQ, ['x%d' % n for n in primes(40)])
  sage: R.inject_variables()
  Defining x2, x3, x5, x7, x11, x13, x17, x19, x23, x29, x31, x37

Sage example in ./sol/mpoly.tex, line 116::

  sage: R.<x,y,z> = QQ[]
  sage: J = R.ideal(x^2*y*z-18, x*y^3*z-24, x*y*z^4-6)
  sage: J.variety(AA)
  [{z: 1, y: 2, x: 3}]

Sage example in ./sol/mpoly.tex, line 124::

  sage: V = J.variety(QQbar)
  sage: [u for u in V if all(a in AA for a in u.values())]
  [{z: 1, y: 2, x: 3}]

Sage example in ./sol/mpoly.tex, line 137::

  sage: M = matrix([ [p.degree(v) for v in (x,y,z)]
  ....:              for p in J.gens()]); M
  [2 1 1]
  [1 3 1]
  [1 1 4]

Sage example in ./sol/mpoly.tex, line 146::

  sage: M.det()
  17

Sage example in ./sol/mpoly.tex, line 154::

  sage: M.change_ring(GF(17)).right_kernel()
  Vector space of degree 3 and dimension 1 over Finite Field of size 17
  Basis matrix:
  [1 9 6]

Sage example in ./sol/mpoly.tex, line 166::

  sage: L.<a> = QQ[sqrt(2-sqrt(3))]; L
    Number Field in a with defining polynomial x^4 - 4*x^2 + 1 with a = 0.5176380902050415?
  sage: R.<x,y> = QQ[]
  sage: J1 = (x^2 + y^2 - 1, 16*x^2*y^2 - 1)*R
  sage: sorted(J1.variety(L), key=str)
  [{y: -1/2*a, x: -1/2*a^3 + 2*a},
   {y: -1/2*a, x: 1/2*a^3 - 2*a},
   {y: -1/2*a^3 + 2*a, x: -1/2*a},
   {y: -1/2*a^3 + 2*a, x: 1/2*a},
   {y: 1/2*a, x: -1/2*a^3 + 2*a},
   {y: 1/2*a, x: 1/2*a^3 - 2*a},
   {y: 1/2*a^3 - 2*a, x: -1/2*a},
   {y: 1/2*a^3 - 2*a, x: 1/2*a}]

Sage example in ./sol/mpoly.tex, line 191::

  sage: R.<x,y> = QQ[]; J2 = (x^2+y^2-1, 4*x^2*y^2-1)*R
  sage: basis = J2.normal_basis(); basis
  [x*y^3, y^3, x*y^2, y^2, x*y, y, x, 1]

Sage example in ./sol/mpoly.tex, line 200::

  sage: xbasis = [(x*p).reduce(J2) for p in basis]; xbasis
  [1/4*y, x*y^3, 1/4, x*y^2, -y^3 + y, x*y, -y^2 + 1, x]
  sage: mat = matrix([ [xp[q] for q in basis]
  ....:                       for xp in xbasis])
  sage: mat
  [  0   0   0   0   0 1/4   0   0]
  [  1   0   0   0   0   0   0   0]
  [  0   0   0   0   0   0   0 1/4]
  [  0   0   1   0   0   0   0   0]
  [  0  -1   0   0   0   1   0   0]
  [  0   0   0   0   1   0   0   0]
  [  0   0   0  -1   0   0   0   1]
  [  0   0   0   0   0   0   1   0]

Sage example in ./sol/mpoly.tex, line 219::

  sage: charpoly = mat.characteristic_polynomial(); charpoly
  x^8 - 2*x^6 + 3/2*x^4 - 1/2*x^2 + 1/16
  sage: solve(SR(charpoly), SR(x))
  [x == -1/2*sqrt(2), x == 1/2*sqrt(2)]

Sage example in ./sol/mpoly.tex, line 278::

  sage: R.<s, c, u, v> = PolynomialRing(QQ, order='lex')
  sage: Rel = ideal(u-(s+c), v-(2*s*c+c^2-s^2), s^2+c^2-1)
  sage: Rel.reduce(s^6)
  1/16*u^2*v^2 - 3/8*u^2*v + 7/16*u^2 + 1/8*v^2 - 1/8*v - 1/8
"""
